"""Return current version, this should be removed."""
# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

__version__ = "1.2.1"


def get_version():
    """Return version."""
    return __version__



"""MobileNet-V3-large-1 for multi-class config."""

# pylint: disable=invalid-name

_base_ = ["../../../../recipes/stages/classification/incremental.yaml", "../base/models/mobilenet_v3.py"]

model = dict(
    type="SAMImageClassifier",
    task="classification",
    backbone=dict(mode="large"),
    head=dict(
        type="CustomNonLinearClsHead",
        in_channels=960,
        hid_channels=1280,
        loss=dict(
            type="CrossEntropyLoss",
            loss_weight=1.0,
        ),
    ),
)

fp16 = dict(loss_scale=512.0)



"""MMOVSSDNeck class for OMZ models."""
# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from typing import Dict, List, Optional, Union

import openvino.runtime as ov
import torch
from mmcv.cnn import ConvModule, DepthwiseSeparableConvModule
from mmcv.runner import BaseModule
from mmdet.models.builder import NECKS
from torch import nn

from otx.core.ov.models.mmov_model import MMOVModel

# pylint: disable=too-many-arguments, too-many-locals


# FIXME: get rid of defined SSDNeck as this is a workaround for forked/deprecated mmdet
class SSDNeck(BaseModule):
    """Extra layers of SSD backbone to generate multi-scale feature maps.

    Args:
        in_channels (Sequence[int]): Number of input channels per scale.
        out_channels (Sequence[int]): Number of output channels per scale.
        level_strides (Sequence[int]): Stride of 3x3 conv per level.
        level_paddings (Sequence[int]): Padding size of 3x3 conv per level.
        l2_norm_scale (float|None): L2 normalization layer init scale.
            If None, not use L2 normalization on the first input feature.
        last_kernel_size (int): Kernel size of the last conv layer.
            Default: 3.
        use_depthwise (bool): Whether to use DepthwiseSeparableConv.
            Default: False.
        conv_cfg (dict): Config dict for convolution layer. Default: None.
        norm_cfg (dict): Dictionary to construct and config norm layer.
            Default: None.
        act_cfg (dict): Config dict for activation layer.
            Default: dict(type='ReLU').
        init_cfg (dict or list[dict], optional): Initialization config dict.
    """

    def __init__(
        self,
        in_channels,
        out_channels,
        level_strides,
        level_paddings,
        l2_norm_scale=20.0,
        last_kernel_size=3,
        use_depthwise=False,
        conv_cfg=None,
        norm_cfg=None,
        act_cfg=None,
        init_cfg=None,
    ):
        if init_cfg is None:
            init_cfg = [
                dict(type="Xavier", distribution="uniform", layer="Conv2d"),
                dict(type="Constant", val=1, layer="BatchNorm2d"),
            ]
        super().__init__(init_cfg)
        assert len(out_channels) > len(in_channels)
        assert len(out_channels) - len(in_channels) == len(level_strides)
        assert len(level_strides) == len(level_paddings)
        assert in_channels == out_channels[: len(in_channels)]

        act_cfg = dict(type="ReLU") if act_cfg is None else act_cfg

        if l2_norm_scale:
            self.l2_norm = L2Norm(in_channels[0], l2_norm_scale)
            self.init_cfg += [dict(type="Constant", val=self.l2_norm.scale, override=dict(name="l2_norm"))]

        self.extra_layers = nn.ModuleList()
        extra_layer_channels = out_channels[len(in_channels) :]
        second_conv = DepthwiseSeparableConvModule if use_depthwise else ConvModule

        for i, (out_channel, stride, padding) in enumerate(zip(extra_layer_channels, level_strides, level_paddings)):
            kernel_size = last_kernel_size if i == len(extra_layer_channels) - 1 else 3
            per_lvl_convs = nn.Sequential(
                ConvModule(
                    out_channels[len(in_channels) - 1 + i],
                    out_channel // 2,
                    1,
                    conv_cfg=conv_cfg,
                    norm_cfg=norm_cfg,
                    act_cfg=act_cfg,
                ),
                second_conv(
                    out_channel // 2,
                    out_channel,
                    kernel_size,
                    stride=stride,
                    padding=padding,
                    conv_cfg=conv_cfg,
                    norm_cfg=norm_cfg,
                    act_cfg=act_cfg,
                ),
            )
            self.extra_layers.append(per_lvl_convs)

    def forward(self, inputs):
        """Forward function."""
        outs = list(inputs)
        if hasattr(self, "l2_norm"):
            outs[0] = self.l2_norm(outs[0])

        feat = outs[-1]
        for layer in self.extra_layers:
            feat = layer(feat)
            outs.append(feat)
        return tuple(outs)


class L2Norm(nn.Module):
    """L2 normalization class."""

    def __init__(self, n_dims, scale=20.0, eps=1e-10):
        """L2 normalization layer.

        Args:
            n_dims (int): Number of dimensions to be normalized
            scale (float, optional): Defaults to 20..
            eps (float, optional): Used to avoid division by zero.
                Defaults to 1e-10.
        """
        super().__init__()
        self.n_dims = n_dims
        self.weight = nn.Parameter(torch.Tensor(self.n_dims))
        self.eps = eps
        self.scale = scale

    def forward(self, x):
        """Forward function."""
        # normalization layer convert to FP32 in FP16 training
        x_float = x.float()
        norm = x_float.pow(2).sum(1, keepdim=True).sqrt() + self.eps
        return (self.weight[None, :, None, None].float().expand_as(x_float) * x_float / norm).type_as(x)


@NECKS.register_module()
class MMOVSSDNeck(SSDNeck):
    """MMOVSSDNeck class for OMZ models."""

    def __init__(
        self,
        model_path_or_model: Union[str, ov.Model],
        weight_path: Optional[str] = None,
        inputs: Optional[Union[Dict[str, Union[str, List[str]]], List[str], str]] = None,
        outputs: Optional[Union[Dict[str, Union[str, List[str]]], List[str], str]] = None,
        init_weight: bool = False,
        verify_shape: bool = True,
    ):
        # dummy
        in_channels = (512, 1024)
        out_channels = (512, 1024, 512, 256, 256, 256)
        level_strides = (2, 2, 1, 1)
        level_paddings = (1, 1, 0, 0)
        l2_norm_scale = None
        super().__init__(
            in_channels=in_channels,
            out_channels=out_channels,
            level_strides=level_strides,
            level_paddings=level_paddings,
            l2_norm_scale=l2_norm_scale,
        )

        self._model_path_or_model = model_path_or_model
        self._weight_path = weight_path
        self._init_weight = init_weight

        self.extra_layers = torch.nn.ModuleList()

        # TODO: Need to fix what exactly the types of inputs and outputs are.
        if not isinstance(inputs, dict) or not isinstance(outputs, dict):
            raise ValueError("The type of inputs & outputs is invalid.")
        for input_e, output_e in zip(inputs["extra_layers"], outputs["extra_layers"]):
            if input_e and output_e:
                self.extra_layers.append(
                    MMOVModel(
                        model_path_or_model,
                        weight_path,
                        inputs=input_e,
                        outputs=output_e,
                        remove_normalize=False,
                        merge_bn=False,
                        paired_bn=False,
                        init_weight=init_weight,
                        verify_shape=verify_shape,
                    )
                )
            else:
                self.extra_layers.append(torch.nn.Identity())

        if "l2_norm" in inputs and "l2_norm" in outputs:
            for input_l2, output_l2 in zip(inputs["l2_norm"], outputs["l2_norm"]):
                self.l2_norm = MMOVModel(
                    model_path_or_model,
                    weight_path,
                    inputs=input_l2,
                    outputs=output_l2,
                    remove_normalize=False,
                    merge_bn=False,
                    paired_bn=False,
                    init_weight=init_weight,
                    verify_shape=verify_shape,
                )

    def init_weights(self, pretrained=None):  # pylint: disable=unused-argument
        """Initial weights of MMOVSSDNeck."""
        # TODO
        return



"""Unit test for otx.algorithms.common.adapters.mmcv.hooks.no_bias_decay_hook."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import torch
from mmcv.utils import Config

from otx.algorithms.common.adapters.mmcv.hooks.no_bias_decay_hook import NoBiasDecayHook
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockRunner:
    def __init__(self):
        self.model = torch.nn.Sequential(
            torch.nn.Conv2d(3, 8, 3),
            torch.nn.BatchNorm2d(8),
        )
        self.optimizer = Config(
            {
                "param_groups": [
                    {
                        "params": None,
                        "weight_decay": None,
                        "lr": 0.01,
                    }
                ]
            }
        )


class TestNoBiasDecayHook:
    @e2e_pytest_unit
    def test_before_train_epoch(self):
        """Test before_train_epoch function."""

        hook = NoBiasDecayHook()
        runner = MockRunner()
        hook.before_train_epoch(runner)
        assert len(runner.optimizer.param_groups) == 3
        assert runner.optimizer.param_groups[0]["params"] is not None
        assert runner.optimizer.param_groups[1]["params"] is not None
        assert runner.optimizer.param_groups[1]["weight_decay"] == 0.0
        assert runner.optimizer.param_groups[1]["lr"] == 0.02
        assert runner.optimizer.param_groups[2]["params"] is not None
        assert runner.optimizer.param_groups[2]["weight_decay"] == 0.0

    @e2e_pytest_unit
    def test_after_train_epoch(self):
        hook = NoBiasDecayHook()
        runner = MockRunner()
        hook.after_train_epoch(runner)
        assert runner.optimizer.param_groups[0]["params"] is not None



"""Unit-Test case for otx.core.data.adapter.anomaly_dataset_adapter."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import os

from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.model_template import TaskType
from otx.api.entities.subset import Subset
from otx.core.data.adapter.anomaly_dataset_adapter import (
    AnomalyClassificationDatasetAdapter,
    AnomalyDetectionDatasetAdapter,
    AnomalySegmentationDatasetAdapter,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.core.data.test_helpers import (
    TASK_NAME_TO_DATA_ROOT,
    TASK_NAME_TO_TASK_TYPE,
)


class TestOTXAnomalyClassificationDatasetAdapter:
    def setup_method(self):
        self.root_path = os.getcwd()
        task = "anomaly_classification"

        self.task_type: TaskType = TASK_NAME_TO_TASK_TYPE[task]
        data_root_dict: dict = TASK_NAME_TO_DATA_ROOT[task]

        self.train_data_roots: str = os.path.join(self.root_path, data_root_dict["train"])
        self.val_data_roots: str = os.path.join(self.root_path, data_root_dict["val"])
        self.test_data_roots: str = os.path.join(self.root_path, data_root_dict["test"])

        self.train_dataset_adapter = AnomalyClassificationDatasetAdapter(
            task_type=self.task_type,
            train_data_roots=self.train_data_roots,
            val_data_roots=self.val_data_roots,
        )

        self.test_dataset_adapter = AnomalyClassificationDatasetAdapter(
            task_type=self.task_type,
            test_data_roots=self.test_data_roots,
        )

    @e2e_pytest_unit
    def test_init(self):
        assert Subset.TRAINING in self.train_dataset_adapter.dataset
        assert Subset.VALIDATION in self.train_dataset_adapter.dataset
        assert Subset.TESTING in self.test_dataset_adapter.dataset

    @e2e_pytest_unit
    def test_get_otx_dataset(self):
        assert isinstance(self.train_dataset_adapter.get_otx_dataset(), DatasetEntity)
        assert isinstance(self.test_dataset_adapter.get_otx_dataset(), DatasetEntity)


class TestOTXAnomalyDetectionDatasetAdapter:
    def setup_method(self):
        self.root_path = os.getcwd()
        task = "anomaly_detection"

        self.task_type: TaskType = TASK_NAME_TO_TASK_TYPE[task]
        data_root_dict: dict = TASK_NAME_TO_DATA_ROOT[task]

        self.train_data_roots: str = os.path.join(self.root_path, data_root_dict["train"])
        self.val_data_roots: str = os.path.join(self.root_path, data_root_dict["val"])
        self.test_data_roots: str = os.path.join(self.root_path, data_root_dict["test"])

        self.train_dataset_adapter = AnomalyDetectionDatasetAdapter(
            task_type=self.task_type,
            train_data_roots=self.train_data_roots,
            val_data_roots=self.val_data_roots,
        )

        self.test_dataset_adapter = AnomalyDetectionDatasetAdapter(
            task_type=self.task_type,
            test_data_roots=self.test_data_roots,
        )

    @e2e_pytest_unit
    def test_init(self):
        assert Subset.TRAINING in self.train_dataset_adapter.dataset
        assert Subset.VALIDATION in self.train_dataset_adapter.dataset
        assert Subset.TESTING in self.test_dataset_adapter.dataset

    @e2e_pytest_unit
    def test_get_otx_dataset(self):
        assert isinstance(self.train_dataset_adapter.get_otx_dataset(), DatasetEntity)
        assert isinstance(self.test_dataset_adapter.get_otx_dataset(), DatasetEntity)


class TestOTXAnomalySegmentationDatasetAdapter:
    def setup_method(self):
        self.root_path = os.getcwd()
        task = "anomaly_segmentation"

        self.task_type: TaskType = TASK_NAME_TO_TASK_TYPE[task]
        data_root_dict: dict = TASK_NAME_TO_DATA_ROOT[task]

        self.train_data_roots: str = os.path.join(self.root_path, data_root_dict["train"])
        self.val_data_roots: str = os.path.join(self.root_path, data_root_dict["val"])
        self.test_data_roots: str = os.path.join(self.root_path, data_root_dict["test"])

        self.train_dataset_adapter = AnomalySegmentationDatasetAdapter(
            task_type=self.task_type,
            train_data_roots=self.train_data_roots,
            val_data_roots=self.val_data_roots,
        )

        self.test_dataset_adapter = AnomalySegmentationDatasetAdapter(
            task_type=self.task_type,
            test_data_roots=self.test_data_roots,
        )

    @e2e_pytest_unit
    def test_init(self):
        assert Subset.TRAINING in self.train_dataset_adapter.dataset
        assert Subset.VALIDATION in self.train_dataset_adapter.dataset
        assert Subset.TESTING in self.test_dataset_adapter.dataset

    @e2e_pytest_unit
    def test_get_otx_dataset(self):
        assert isinstance(self.train_dataset_adapter.get_otx_dataset(), DatasetEntity)
        assert isinstance(self.test_dataset_adapter.get_otx_dataset(), DatasetEntity)



import argparse
import tempfile

import pytest
from omegaconf import DictConfig, OmegaConf

from otx.cli.manager.config_manager import (
    DEFAULT_MODEL_TEMPLATE_ID,
    ConfigManager,
    set_workspace,
)
from otx.cli.registry import Registry
from otx.cli.utils.errors import (
    CliException,
    ConfigValueError,
    FileNotExistError,
    NotSupportedError,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


def test_set_workspace():
    task = "CLASSIFICATION"
    root = "/home/user/data"
    name = "otx-workspace"

    expected_path = f"{root}/{name}-{task}"
    assert set_workspace(task, root, name) == expected_path

    expected_path = f"./{name}-{task}"
    assert set_workspace(task, name=name) == expected_path


class TestConfigManager:
    @pytest.fixture
    def config_manager(self, mocker):
        args = mocker.MagicMock()
        args.template = "."
        args.config_path = "path/to/config.yaml"
        args.workspace_path = "path/to/workspace"
        args.mode = "train"
        args.task_type = "classification"
        args.train_type = "incremental"
        return ConfigManager(args)

    def get_default_template(self, otx_root, task_type):
        otx_registry = Registry(otx_root).filter(task_type=task_type)
        return otx_registry.get(DEFAULT_MODEL_TEMPLATE_ID[task_type.upper()])

    @e2e_pytest_unit
    def test_check_workspace(self, mocker, config_manager):
        mock_exists = mocker.patch("otx.cli.manager.config_manager.Path.exists")
        # Define the return value of the `os.path.exists` function
        mock_exists.return_value = True
        # Call the function and check the returned value
        assert config_manager.check_workspace()
        mock_exists.return_value = False
        assert not config_manager.check_workspace()

    @e2e_pytest_unit
    def test_get_arg_data_yaml(self, mocker):
        # Call the function and check the returned value
        args = mocker.MagicMock()
        args.template = "."
        args.train_data_roots = "path/to/data/train"
        args.val_data_roots = "path/to/data/val"
        args.test_data_roots = "path/to/data/test"
        args.unlabeled_data_roots = None
        args.mode = "train"
        config_manager = ConfigManager(args)
        assert config_manager._get_arg_data_yaml() == {
            "data": {
                "train": {"ann-files": None, "data-roots": "path/to/data/train"},
                "val": {"ann-files": None, "data-roots": "path/to/data/val"},
                "test": {"ann-files": None, "data-roots": None},
                "unlabeled": {"file-list": None, "data-roots": None},
            }
        }
        config_manager.mode = "test"
        assert config_manager._get_arg_data_yaml() == {
            "data": {
                "train": {"ann-files": None, "data-roots": None},
                "val": {"ann-files": None, "data-roots": None},
                "test": {"ann-files": None, "data-roots": "path/to/data/test"},
                "unlabeled": {"file-list": None, "data-roots": None},
            }
        }

        args.unlabeled_data_roots = "path/to/data/unlabeled"
        config_manager = ConfigManager(args)
        assert config_manager._get_arg_data_yaml() == {
            "data": {
                "train": {"ann-files": None, "data-roots": "path/to/data/train"},
                "val": {"ann-files": None, "data-roots": "path/to/data/val"},
                "test": {"ann-files": None, "data-roots": None},
                "unlabeled": {"file-list": None, "data-roots": "path/to/data/unlabeled"},
            }
        }

    @e2e_pytest_unit
    def test_create_empty_data_cfg(self, config_manager):
        # Call the function and check the returned value
        assert config_manager._create_empty_data_cfg() == {
            "data": {
                "train": {"ann-files": None, "data-roots": None},
                "val": {"ann-files": None, "data-roots": None},
                "test": {"ann-files": None, "data-roots": None},
                "unlabeled": {"file-list": None, "data-roots": None},
            }
        }

    @e2e_pytest_unit
    def test_export_data_cfg(self, mocker, config_manager):
        # Mock data
        data_cfg = {
            "data": {
                "train": {"ann-files": "path/to/train/ann", "data-roots": "path/to/train/images"},
                "val": {"ann-files": "path/to/val/ann", "data-roots": "path/to/val/images"},
                "test": {"ann-files": "path/to/test/ann", "data-roots": "path/to/test/images"},
                "unlabeled": {"file-list": "path/to/unlabeled/files", "data-roots": "path/to/unlabeled/images"},
            }
        }

        # Create temporary file
        with tempfile.NamedTemporaryFile(mode="w", delete=False) as temp_file:
            output_path = temp_file.name

        # Mock write_text function
        mock_write_text = mocker.patch("pathlib.Path.write_text")

        # Test the function
        config_manager._export_data_cfg(data_cfg, output_path)

        # Assertions
        mock_write_text.assert_called_once_with(OmegaConf.to_yaml(data_cfg), encoding="utf-8")

    @e2e_pytest_unit
    def test_build_workspace(self, mocker):
        # Setup
        task_type = "CLASSIFICATION"
        train_type = "Semisupervised"
        workspace_path = "./otx-workspace"
        args = mocker.Mock()
        args.autosplit = None
        args.workspace = workspace_path
        config_manager = ConfigManager(args)
        template = self.get_default_template(config_manager.otx_root, task_type)
        config_manager.template = template
        config_manager.train_type = train_type
        config_manager.task_type = task_type

        pathlib_mkdir_mock = mocker.patch("pathlib.Path.mkdir")
        pathlib_exists_mock = mocker.patch("pathlib.Path.exists", return_value=True)

        set_workspace_mock = mocker.patch("otx.cli.manager.config_manager.set_workspace", return_value=workspace_path)

        parse_model_template_mock = mocker.patch(
            "otx.cli.manager.config_manager.parse_model_template", return_value=template
        )

        gen_params_dict_from_args_mock = mocker.patch(
            "otx.cli.manager.config_manager.gen_params_dict_from_args", return_value={}
        )
        omageconf_load_mock = mocker.patch("otx.cli.manager.config_manager.OmegaConf.load", return_value=template)
        omageconf_merge_mock = mocker.patch("otx.cli.manager.config_manager.OmegaConf.merge", return_value=template)
        path_write_text_mock = mocker.patch("otx.cli.manager.config_manager.Path.write_text")

        config_manager_check_workspace_mock = mocker.patch(
            "otx.cli.manager.config_manager.ConfigManager.check_workspace", return_value=False
        )

        config_manager_copy_config_files_mock = mocker.patch(
            "otx.cli.manager.config_manager.ConfigManager._copy_config_files"
        )
        shutil_copyfile_mock = mocker.patch("shutil.copyfile")

        # Run
        config_manager.build_workspace()

        # Check
        set_workspace_mock.assert_called_once_with(task=task_type)
        parse_model_template_mock.assert_called_once_with(str(config_manager.workspace_root / "template.yaml"))

        # Calls
        pathlib_mkdir_mock.assert_called()
        pathlib_exists_mock.assert_called()

        gen_params_dict_from_args_mock.assert_called_once_with(args)
        omageconf_load_mock.assert_called()
        omageconf_merge_mock.assert_called()
        path_write_text_mock.assert_called()

        config_manager_check_workspace_mock.assert_called()
        config_manager_copy_config_files_mock.assert_called()

        shutil_copyfile_mock.assert_called()

    @e2e_pytest_unit
    def test_update_data_config(self, config_manager, tmp_dir_path):
        data_yaml = {
            "data": {
                "train": {"data-roots": "/path/to/train/data"},
                "val": {"data-roots": "/path/to/val/data"},
                "test": {"data-roots": "/path/to/test/data"},
                "unlabeled": {"file-list": "/path/to/unlabeled/filelist", "data-roots": "/path/to/unlabeled/data"},
            }
        }
        data_yaml_path = tmp_dir_path / "data.yaml"
        OmegaConf.save(data_yaml, str(data_yaml_path))

        config_manager.update_data_config(OmegaConf.load(str(data_yaml_path)))
        assert config_manager.data_config == {
            "train_subset": {"data_root": "/path/to/train/data"},
            "val_subset": {"data_root": "/path/to/val/data"},
            "test_subset": {"data_root": "/path/to/test/data"},
            "unlabeled_subset": {
                "data_root": "/path/to/unlabeled/data",
                "file_list": "/path/to/unlabeled/filelist",
            },
        }

        data_yaml["data"]["train"]["data-roots"] = None
        data_yaml_path = tmp_dir_path / "data.yaml"
        OmegaConf.save(data_yaml, str(data_yaml_path))

        config_manager.update_data_config(OmegaConf.load(str(data_yaml_path)))
        assert config_manager.data_config == {
            "train_subset": {"data_root": "/path/to/train/data"},
            "val_subset": {"data_root": "/path/to/val/data"},
            "test_subset": {"data_root": "/path/to/test/data"},
            "unlabeled_subset": {
                "data_root": "/path/to/unlabeled/data",
                "file_list": "/path/to/unlabeled/filelist",
            },
        }

        data_yaml["data"].pop("unlabeled")
        data_yaml_path = tmp_dir_path / "data.yaml"
        OmegaConf.save(data_yaml, str(data_yaml_path))

        config_manager.update_data_config(OmegaConf.load(str(data_yaml_path)))
        assert config_manager.data_config == {
            "train_subset": {"data_root": "/path/to/train/data"},
            "val_subset": {"data_root": "/path/to/val/data"},
            "test_subset": {"data_root": "/path/to/test/data"},
            "unlabeled_subset": {
                "data_root": "/path/to/unlabeled/data",
                "file_list": "/path/to/unlabeled/filelist",
            },
        }

    @e2e_pytest_unit
    def test_get_hyparams_config(self, mocker):
        mock_hyper_parameters = {
            "learning_rate": {
                "type": "FLOAT",
                "default_value": "0.01",
                "max_value": "1.0",
                "min_value": "0.0",
                "affects_outcome_of": ["TRAINING"],
            },
            "batch_size": {
                "type": "INTEGER",
                "default_value": "16",
                "max_value": "128",
                "min_value": "1",
                "affects_outcome_of": ["TRAINING", "TESTING"],
            },
        }
        mock_template = DictConfig({"hyper_parameters": DictConfig({"data": mock_hyper_parameters})})

        parser = argparse.ArgumentParser()
        parser.add_argument("--template")
        parser.add_argument(
            "--learning_rate",
            dest="params.learning_rate",
        )
        parser.add_argument(
            "--batch_size",
            dest="params.batch_size",
        )
        mock_input = ["--learning_rate", "0.5", "--batch_size", "8"]
        mock_args = parser.parse_args(mock_input)
        expected_hyper_parameters = {
            "learning_rate": {
                "type": "FLOAT",
                "default_value": "0.01",
                "max_value": "1.0",
                "min_value": "0.0",
                "affects_outcome_of": ["TRAINING"],
                "value": "0.5",
            },
            "batch_size": {
                "type": "INTEGER",
                "default_value": "16",
                "max_value": "128",
                "min_value": "1",
                "affects_outcome_of": ["TRAINING", "TESTING"],
                "value": "8",
            },
        }
        mock_create = mocker.patch("otx.cli.manager.config_manager.create", return_value=expected_hyper_parameters)

        config_manager = ConfigManager(mock_args)
        config_manager.template = mock_template
        config_manager.get_hyparams_config()
        mock_create.assert_called_once_with(expected_hyper_parameters)

    @e2e_pytest_unit
    def test_data_config_file_path(self, mocker, tmp_dir_path):
        parser = argparse.ArgumentParser()
        parser.add_argument("--template")
        parser.add_argument("--data")
        args = parser.parse_args([])
        config_manager = ConfigManager(args)

        # set up test workspace
        workspace_root = tmp_dir_path / "test_data_config"
        config_manager.workspace_root = workspace_root
        # workspace_root.mkdir(exist_ok=True, parents=True)
        assert config_manager.data_config_file_path == workspace_root / "data.yaml"

        # expected file path
        mock_exists = mocker.patch("otx.cli.manager.config_manager.Path.exists", return_value=False)
        expected_file_path = tmp_dir_path / "data.yaml"
        args = parser.parse_args(["--data", str(expected_file_path)])
        config_manager.args = args
        with pytest.raises(FileNotExistError):
            config_manager.data_config_file_path

        mock_exists.return_value = True
        assert config_manager.data_config_file_path == expected_file_path

    @e2e_pytest_unit
    def test_configure_template(self, mocker):
        # Given
        mock_args = mocker.MagicMock()
        mock_args.train_data_roots = ["/path/to/train/data"]
        mock_args.template = None
        mock_workspace_root = mocker.MagicMock()
        mock_workspace_root.exists.return_value = True

        mock_template = DictConfig({"name": "template_name", "task_type": "CLASSIFICATION"})
        mock_check_workspace = mocker.patch(
            "otx.cli.manager.config_manager.ConfigManager.check_workspace", return_value=True
        )
        mocker.patch("otx.cli.manager.config_manager.ConfigManager._get_template", return_value=mock_template)
        mocker.patch("otx.cli.manager.config_manager.ConfigManager._get_train_type", return_value="Incremental")
        mock_parse_model_template = mocker.patch(
            "otx.cli.manager.config_manager.parse_model_template", return_value=mock_template
        )

        config_manager = ConfigManager(args=mock_args, workspace_root=mock_workspace_root)

        # When
        config_manager.configure_template()

        # Then
        assert config_manager.task_type == "CLASSIFICATION"
        assert config_manager.model == "template_name"
        assert config_manager.train_type == "Incremental"

        config_manager.mode = "build"
        mocker.patch("otx.cli.manager.config_manager.ConfigManager._check_rebuild", return_value=True)
        config_manager.configure_template()
        assert config_manager.rebuild
        assert config_manager.task_type == "CLASSIFICATION"
        assert config_manager.model == "template_name"
        assert config_manager.train_type == "Incremental"

        mock_check_workspace.return_value = False
        mocker.patch("pathlib.Path.exists", return_value=True)
        config_manager.template = "test/template"
        config_manager.configure_template()
        mock_parse_model_template.assert_called_with("test/template")

        config_manager.template = None
        config_manager.task_type = None
        mock_check_workspace = mocker.patch(
            "otx.cli.manager.config_manager.ConfigManager.auto_task_detection", return_value="CLASSIFICATION"
        )
        config_manager.configure_template()
        assert config_manager.task_type == "CLASSIFICATION"
        assert config_manager.model == "template_name"
        assert config_manager.train_type == "Incremental"

    @e2e_pytest_unit
    def test__check_rebuild(self, mocker):
        mock_template = mocker.MagicMock()
        mock_template.task_type = "CLASSIFICATION"

        mock_args = mocker.MagicMock()
        mock_args.mode = "build"
        mock_args.task = "DETECTION"
        mock_args.template = mock_template

        config_manager = ConfigManager(mock_args)
        with pytest.raises(NotSupportedError):
            config_manager._check_rebuild()

        config_manager.template.task_type = "DETECTION"
        config_manager.args.model = None
        config_manager.args.train_type = ""
        assert not config_manager._check_rebuild()

        config_manager.args.model = "SSD"
        config_manager.template.name = "ATSS"
        config_manager.args.train_type = "Semisupervised"
        assert config_manager._check_rebuild()

    @e2e_pytest_unit
    def test_configure_data_config(self, mocker):
        data_yaml = {
            "data": {
                "train": {"ann-files": None, "data-roots": "train/data/roots"},
                "val": {"ann-files": None, "data-roots": None},
                "test": {"ann-files": None, "data-roots": None},
                "unlabeled": {"file-list": None, "data-roots": None},
            }
        }
        mock_configure_dataset = mocker.patch(
            "otx.cli.manager.config_manager.configure_dataset", return_value=data_yaml
        )
        mock_auto_split = mocker.patch("otx.cli.manager.config_manager.ConfigManager.auto_split_data", return_value={})
        mock_get_data_yaml = mocker.patch(
            "otx.cli.manager.config_manager.ConfigManager._get_arg_data_yaml", return_value=data_yaml
        )
        mock_save_data = mocker.patch("otx.cli.manager.config_manager.ConfigManager._save_data")
        mock_export_data_cfg = mocker.patch("otx.cli.manager.config_manager.ConfigManager._export_data_cfg")
        mock_update_data_config = mocker.patch("otx.cli.manager.config_manager.ConfigManager.update_data_config")

        mock_args = mocker.MagicMock()
        mock_args.mode = "build"

        config_manager = ConfigManager(mock_args)
        config_manager.train_type = "Incremental"
        config_manager.configure_data_config(update_data_yaml=True)

        mock_configure_dataset.assert_called_once()
        mock_auto_split.assert_called_once_with("train/data/roots", "")
        mock_get_data_yaml.assert_called_once()
        mock_save_data.assert_called_once()
        mock_export_data_cfg.assert_called_once()
        mock_update_data_config.assert_called_once_with(data_yaml)

    @e2e_pytest_unit
    def test__get_train_type(self, mocker):
        mock_args = mocker.MagicMock()
        mock_params_dict = {"algo_backend": {"train_type": {"value": "Semisupervised"}}}
        mock_configure_dataset = mocker.patch(
            "otx.cli.manager.config_manager.gen_params_dict_from_args", return_value=mock_params_dict
        )
        config_manager = ConfigManager(args=mock_args)
        config_manager.mode = "build"
        assert config_manager._get_train_type() == "Semisupervised"

        config_manager.args.train_type = "Incremental"
        mock_configure_dataset.return_value = {}
        assert config_manager._get_train_type() == "Incremental"

        mock_template = mocker.MagicMock()
        mock_template.hyper_parameters.parameter_overrides = {
            "algo_backend": {"train_type": {"default_value": "Selfsupervised"}}
        }
        config_manager.template = mock_template
        assert config_manager._get_train_type(ignore_args=True) == "Selfsupervised"

        config_manager.template.hyper_parameters.parameter_overrides = {}
        assert config_manager._get_train_type(ignore_args=True) == "Incremental"

    @e2e_pytest_unit
    def test_auto_task_detection(self, mocker):
        mock_args = mocker.MagicMock()
        config_manager = ConfigManager(args=mock_args)
        with pytest.raises(CliException):
            config_manager.auto_task_detection("")

        mock_get_data_format = mocker.patch(
            "otx.cli.manager.config_manager.DatasetManager.get_data_format", return_value="Unexpected"
        )
        with pytest.raises(ConfigValueError):
            config_manager.auto_task_detection("data/roots")

        mock_get_data_format.return_value = "coco"
        assert config_manager.auto_task_detection("data/roots") == "DETECTION"

    @e2e_pytest_unit
    def test_auto_split_data(self, mocker):
        mock_get_data_format = mocker.patch(
            "otx.cli.manager.config_manager.DatasetManager.get_data_format", return_value="coco"
        )
        mock_import_dataset = mocker.patch(
            "otx.cli.manager.config_manager.DatasetManager.import_dataset", return_value=None
        )
        mock_get_train_dataset = mocker.patch(
            "otx.cli.manager.config_manager.DatasetManager.get_train_dataset", return_value="train_dataset"
        )
        mock_get_val_dataset = mocker.patch(
            "otx.cli.manager.config_manager.DatasetManager.get_val_dataset", return_value="val_dataset"
        )
        mock_auto_split = mocker.patch(
            "otx.cli.manager.config_manager.DatasetManager.auto_split",
            return_value={"train": "auto_train", "val": "auto_val"},
        )

        mock_args = mocker.MagicMock()
        config_manager = ConfigManager(args=mock_args)
        assert config_manager.auto_split_data("test_data_root", task="DETECTION") == {
            "train": "train_dataset",
            "val": "val_dataset",
        }

        mock_get_val_dataset.return_value = None
        assert config_manager.auto_split_data("test_data_root", task="DETECTION") == {
            "train": "auto_train",
            "val": "auto_val",
        }

        mock_get_data_format.return_value = "Unexpected"
        assert config_manager.auto_split_data("test_data_root", task="DETECTION") is None
        mock_get_data_format.assert_called()
        mock_import_dataset.assert_called()
        mock_get_train_dataset.assert_called()
        mock_get_val_dataset.assert_called()
        mock_auto_split.assert_called()

    @e2e_pytest_unit
    def test_get_dataset_config(self, mocker):
        mock_args = mocker.MagicMock()
        config_manager = ConfigManager(args=mock_args)
        config_manager.task_type = "DETECTION"
        config_manager.data_config = {
            "train_subset": {"data_root": "train_path"},
            "val_subset": {"data_root": "val_path"},
            "test_subset": {"data_root": "test_path"},
        }
        dataset_config = config_manager.get_dataset_config(["train", "val", "test"])
        assert dataset_config["task_type"] == "DETECTION"
        assert "train_data_roots" in dataset_config
        assert "val_data_roots" in dataset_config
        assert "test_data_roots" in dataset_config



#
# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

""" This module contains tests for the mapper for ID entities """

import pytest

from otx.api.entities.id import ID
from otx.api.serialization.id_mapper import IDMapper
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestIDMapper:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_serialized_representiaton(self):
        """
        This test serializes ID and checks serialized representation.
        """

        id_ = ID("21434231456")
        serialized_id = IDMapper.forward(id_)
        assert serialized_id == "21434231456"

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_serialization_deserialization(self):
        """
        This test serializes ID, deserializes serialized ID and compare with original.
        """

        id_ = ID("21434231456")
        serialized_id = IDMapper.forward(id_)
        deserialized_id = IDMapper.backward(serialized_id)
        assert id_ == deserialized_id



"""OTX Adapters - mmseg.utils."""

# Copyright (C) 2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

from .builder import build_scalar_scheduler, build_segmentor
from .config_utils import (
    patch_config,
    patch_datasets,
    patch_evaluation,
    prepare_for_training,
    set_hyperparams,
)
from .data_utils import get_valid_label_mask_per_batch, load_dataset_items
from .exporter import SegmentationExporter

__all__ = [
    "patch_config",
    "SegmentationExporter",
    "patch_datasets",
    "patch_evaluation",
    "prepare_for_training",
    "set_hyperparams",
    "load_dataset_items",
    "build_scalar_scheduler",
    "build_segmentor",
    "get_valid_label_mask_per_batch",
]



"""Cancel hooks."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#


import os
from typing import Callable

from mmcv.runner import BaseRunner, EpochBasedRunner
from mmcv.runner.hooks import HOOKS, Hook

from otx.algorithms.common.utils.logger import get_logger

logger = get_logger()


# pylint: disable=too-many-instance-attributes, protected-access, too-many-arguments, unused-argument
@HOOKS.register_module()
class CancelTrainingHook(Hook):
    """CancelTrainingHook for Training Stopping."""

    def __init__(self, interval: int = 5):
        """Periodically check whether whether a stop signal is sent to the runner during model training.

        Every 'check_interval' iterations, the work_dir for the runner is checked to see if a file '.stop_training'
        is present. If it is, training is stopped.

        :param interval: Period for checking for stop signal, given in iterations.

        """
        self.interval = interval

    @staticmethod
    def _check_for_stop_signal(runner: BaseRunner):
        """Log _check_for_stop_signal for CancelTrainingHook."""
        work_dir = runner.work_dir
        stop_filepath = os.path.join(work_dir, ".stop_training")
        if os.path.exists(stop_filepath):
            if isinstance(runner, EpochBasedRunner):
                epoch = runner.epoch
                runner._max_epochs = epoch  # Force runner to stop by pretending it has reached it's max_epoch
            runner.should_stop = True  # Set this flag to true to stop the current training epoch
            os.remove(stop_filepath)

    def after_train_iter(self, runner: BaseRunner):
        """Log after_train_iter for CancelTrainingHook."""
        if not self.every_n_iters(runner, self.interval):
            return
        self._check_for_stop_signal(runner)


@HOOKS.register_module()
class CancelInterfaceHook(Hook):
    """Cancel interface. If called, running job will be terminated."""

    def __init__(self, init_callback: Callable, interval=5):
        self.on_init_callback = init_callback
        self.runner = None
        self.interval = interval

    def cancel(self):
        """Cancel."""
        logger.info("CancelInterfaceHook.cancel() is called.")
        if self.runner is None:
            logger.warning("runner is not configured yet. ignored this request.")
            return

        if self.runner.should_stop:
            logger.warning("cancel already requested.")
            return

        if isinstance(self.runner, EpochBasedRunner):
            epoch = self.runner.epoch
            self.runner._max_epochs = epoch  # Force runner to stop by pretending it has reached it's max_epoch
        self.runner.should_stop = True  # Set this flag to true to stop the current training epoch
        logger.info("requested stopping to the runner")

    def before_run(self, runner):
        """Before run."""
        self.runner = runner
        self.on_init_callback(self)

    def after_run(self, runner):
        """After run."""
        self.runner = None



"""Adapters for mmcv support."""

# Copyright (C) 2022-2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

from .adaptive_training_hook import AdaptiveTrainSchedulingHook
from .cancel_hook import CancelInterfaceHook, CancelTrainingHook
from .checkpoint_hook import (
    CheckpointHookWithValResults,
    EnsureCorrectBestCheckpointHook,
    SaveInitialWeightHook,
)
from .composed_dataloaders_hook import ComposedDataLoadersHook
from .custom_model_ema_hook import CustomModelEMAHook, EMAMomentumUpdateHook
from .dual_model_ema_hook import DualModelEMAHook
from .early_stopping_hook import (
    EarlyStoppingHook,
    LazyEarlyStoppingHook,
    ReduceLROnPlateauLrUpdaterHook,
    StopLossNanTrainingHook,
)
from .eval_hook import CustomEvalHook, DistCustomEvalHook
from .force_train_hook import ForceTrainModeHook
from .fp16_sam_optimizer_hook import Fp16SAMOptimizerHook
from .ib_loss_hook import IBLossHook
from .logger_hook import LoggerReplaceHook, OTXLoggerHook
from .lr_updater_hook import CustomstepLrUpdaterHook
from .model_ema_v2_hook import ModelEmaV2Hook
from .no_bias_decay_hook import NoBiasDecayHook
from .progress_hook import OTXProgressHook
from .recording_forward_hook import (
    ActivationMapHook,
    BaseRecordingForwardHook,
    EigenCamHook,
    FeatureVectorHook,
)
from .sam_optimizer_hook import SAMOptimizerHook
from .semisl_cls_hook import SemiSLClsHook
from .task_adapt_hook import TaskAdaptHook
from .two_crop_transform_hook import TwoCropTransformHook
from .unbiased_teacher_hook import UnbiasedTeacherHook

__all__ = [
    "AdaptiveTrainSchedulingHook",
    "CancelInterfaceHook",
    "CancelTrainingHook",
    "CustomstepLrUpdaterHook",
    "CheckpointHookWithValResults",
    "EnsureCorrectBestCheckpointHook",
    "ComposedDataLoadersHook",
    "CustomEvalHook",
    "DistCustomEvalHook",
    "EarlyStoppingHook",
    "LazyEarlyStoppingHook",
    "ReduceLROnPlateauLrUpdaterHook",
    "EMAMomentumUpdateHook",
    "ForceTrainModeHook",
    "Fp16SAMOptimizerHook",
    "StopLossNanTrainingHook",
    "IBLossHook",
    "OTXLoggerHook",
    "LoggerReplaceHook",
    "CustomModelEMAHook",
    "DualModelEMAHook",
    "ModelEmaV2Hook",
    "NoBiasDecayHook",
    "OTXProgressHook",
    "BaseRecordingForwardHook",
    "EigenCamHook",
    "ActivationMapHook",
    "FeatureVectorHook",
    "SAMOptimizerHook",
    "SaveInitialWeightHook",
    "SemiSLClsHook",
    "TaskAdaptHook",
    "TwoCropTransformHook",
    "UnbiasedTeacherHook",
]



"""OTX encoder decoder for semantic segmentation."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import functools

import torch
from mmseg.models import SEGMENTORS
from mmseg.models.segmentors.encoder_decoder import EncoderDecoder
from mmseg.utils import get_root_logger

from otx.algorithms.common.adapters.mmdeploy.utils import is_mmdeploy_enabled
from otx.algorithms.common.utils.task_adapt import map_class_names


# pylint: disable=unused-argument, line-too-long
@SEGMENTORS.register_module()
class OTXEncoderDecoder(EncoderDecoder):
    """OTX encoder decoder."""

    def __init__(self, task_adapt=None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Hook for class-sensitive weight loading
        assert task_adapt is not None, "When using task_adapt, task_adapt must be set."

        self._register_load_state_dict_pre_hook(
            functools.partial(
                self.load_state_dict_pre_hook,
                self,  # model
                task_adapt["dst_classes"],  # model_classes
                task_adapt["src_classes"],  # chkpt_classes
            )
        )

    def simple_test(self, img, img_meta, rescale=True, output_logits=False):
        """Simple test with single image."""
        seg_logit = self.inference(img, img_meta, rescale)
        if output_logits:
            seg_pred = seg_logit
        elif self.out_channels == 1:
            seg_pred = (seg_logit > self.decode_head.threshold).to(seg_logit).squeeze(1)
        else:
            seg_pred = seg_logit.argmax(dim=1)
        if torch.onnx.is_in_onnx_export():
            # our inference backend only support 4D output
            if seg_pred.dim() != 4:
                seg_pred = seg_pred.unsqueeze(0)
            return seg_pred
        seg_pred = seg_pred.cpu().numpy()
        seg_pred = list(seg_pred)
        return seg_pred

    @staticmethod
    def load_state_dict_pre_hook(
        model, model_classes, chkpt_classes, chkpt_dict, prefix, *args, **kwargs
    ):  # pylint: disable=too-many-locals, unused-argument
        """Modify input state_dict according to class name matching before weight loading."""
        logger = get_root_logger("INFO")
        logger.info(f"----------------- OTXEncoderDecoder.load_state_dict_pre_hook() called w/ prefix: {prefix}")

        # Dst to src mapping index
        model_classes = list(model_classes)
        chkpt_classes = list(chkpt_classes)
        model2chkpt = map_class_names(model_classes, chkpt_classes)
        logger.info(f"{chkpt_classes} -> {model_classes} ({model2chkpt})")

        model_dict = model.state_dict()
        param_names = [
            "decode_head.conv_seg.weight",
            "decode_head.conv_seg.bias",
        ]
        for model_name in param_names:
            chkpt_name = prefix + model_name
            if model_name not in model_dict or chkpt_name not in chkpt_dict:
                logger.info(f"Skipping weight copy: {chkpt_name}")
                continue

            # Mix weights
            model_param = model_dict[model_name].clone()
            chkpt_param = chkpt_dict[chkpt_name]
            for model_key, c in enumerate(model2chkpt):
                if c >= 0:
                    model_param[model_key].copy_(chkpt_param[c])

            # Replace checkpoint weight by mixed weights
            chkpt_dict[chkpt_name] = model_param


if is_mmdeploy_enabled():
    from mmdeploy.core import FUNCTION_REWRITER

    from otx.algorithms.common.adapters.mmcv.hooks.recording_forward_hook import (  # pylint: disable=ungrouped-imports
        FeatureVectorHook,
    )

    BASE_CLASS = "otx.algorithms.segmentation.adapters.mmseg.models.segmentors.otx_encoder_decoder.OTXEncoderDecoder"

    @FUNCTION_REWRITER.register_rewriter(f"{BASE_CLASS}.extract_feat")
    def single_stage_detector__extract_feat(ctx, self, img):
        """Extract feature."""
        feat = self.backbone(img)
        self.feature_map = feat
        if self.with_neck:
            feat = self.neck(feat)
        return feat

    @FUNCTION_REWRITER.register_rewriter(f"{BASE_CLASS}.simple_test")
    def single_stage_detector__simple_test(ctx, self, img, img_metas, **kwargs):
        """Test."""
        # with output activation
        seg_logit = self.inference(img, img_metas, True)
        if ctx.cfg["dump_features"]:
            feature_vector = FeatureVectorHook.func(self.feature_map)
            return seg_logit, feature_vector
        return seg_logit



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import pytest
from bson import ObjectId

from otx.api.entities.id import ID
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestID:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_id(self):
        """
        <b>Description:</b>
        Check ID class object initialization

        <b>Input data:</b>
        ID object with specified representation parameter

        <b>Expected results:</b>
        Test passes if ID object representation property and __repr__ method return expected values

        <b>Steps</b>
        1. Check representation property and __repr__ method for ID object with not specified representation parameter
        2. Check representation property and __repr__ method for ID object with ObjectId class representation parameter
        3. Check representation property and __repr__ method for ID object with str type representation parameter
        """
        # Scenario for ID object with not specified representation parameter
        no_representation_id = ID()
        assert no_representation_id.representation == ""
        assert repr(no_representation_id.representation) == "ID()"
        # Scenario for ID object with ObjectId class representation parameter
        expected_oid = "61a8b869fb7665916a39eb95"
        oid_representation = ObjectId(expected_oid)
        oid_representation_id = ID(oid_representation)
        assert oid_representation_id.representation == "61a8b869fb7665916a39eb95"
        assert repr(oid_representation_id.representation) == "ID(61a8b869fb7665916a39eb95)"
        # Scenario for ID object with str-type representation parameter
        str_representation = " String-type representation ID_1 "
        str_representation_id = ID(str_representation)
        # Leading and trailing whitespaces should be removed, only uppercase letters should be replaced by lowercase
        assert str_representation_id.representation == "string-type representation id_1"
        assert repr(str_representation_id) == "ID(string-type representation id_1)"



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-Lggggcense-Identifier: Apache-2.0
#

import pytest
import torch

from otx.core.ov.ops.matmuls import EinsumV7, MatMulV0
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestMatMulV0:
    @pytest.fixture(autouse=True)
    def setup(self):
        self.shape = (10, 3, 5)
        self.input_1 = torch.randn(10, 3, 4)
        self.input_2 = torch.randn(10, 4, 5)

    @e2e_pytest_unit
    def test_forward(self):
        op = MatMulV0("dummy", shape=(self.shape,), transpose_a=False, transpose_b=False)
        output = op(self.input_1, self.input_2)
        assert output.shape == self.shape
        assert torch.equal(output, torch.matmul(self.input_1, self.input_2))

        op = MatMulV0("dummy", shape=(self.shape,), transpose_a=True, transpose_b=False)
        output = op(self.input_1.permute(0, 2, 1), self.input_2)
        assert output.shape == self.shape
        assert torch.equal(output, torch.matmul(self.input_1, self.input_2))

        op = MatMulV0("dummy", shape=(self.shape,), transpose_a=False, transpose_b=True)
        output = op(self.input_1, self.input_2.permute(0, 2, 1))
        assert output.shape == self.shape
        assert torch.equal(output, torch.matmul(self.input_1, self.input_2))

        op = MatMulV0("dummy", shape=(self.shape,), transpose_a=True, transpose_b=True)
        output = op(self.input_1.permute(0, 2, 1), self.input_2.permute(0, 2, 1))
        assert output.shape == self.shape
        assert torch.equal(output, torch.matmul(self.input_1, self.input_2))


class TestEinsumV7:
    @e2e_pytest_unit
    def test_forward(self):
        input = torch.randn(4, 4)
        op = EinsumV7("dummy", shape=(1,), equation="ii")
        output = op(input)
        assert torch.equal(output, torch.einsum("ii", input))

        input_1 = torch.randn(3, 2, 5)
        input_2 = torch.randn(3, 5, 4)
        op = EinsumV7("dummy", shape=(1,), equation="bij,bjk->bik")
        output = op(input_1, input_2)
        assert torch.equal(output, torch.einsum("bij,bjk->bik", input_1, input_2))



"""Module for the MMOVBackbone class."""

from typing import Dict, List, Union

from mmcls.models.builder import BACKBONES

from otx.core.ov.graph.parsers.cls import cls_base_parser
from otx.core.ov.models.mmov_model import MMOVModel


@BACKBONES.register_module()
class MMOVBackbone(MMOVModel):
    """MMOVBackbone class.

    Args:
        *args: positional arguments.
        **kwargs: keyword arguments.
    """

    @staticmethod
    def parser(graph, **kwargs) -> Dict[str, Union[List[str], Dict[str, List[str]]]]:
        """Parses the input and output of the model.

        Args:
            graph: input graph.
            **kwargs: keyword arguments.

        Returns:
            Dictionary containing input and output of the model.
        """
        output = cls_base_parser(graph, "backbone")
        if output is None:
            raise ValueError("Parser can not determine input and output of model. Please provide them explicitly")
        return output

    def init_weights(self, pretrained=None):  # pylint: disable=unused-argument
        """Initializes the weights of the model.

        Args:
            pretrained: pretrained weights. Default: None.
        """
        # TODO
        return



"""Initial file for mmdetection detectors."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from .custom_atss_detector import CustomATSS
from .custom_maskrcnn_detector import CustomMaskRCNN
from .custom_maskrcnn_tile_optimized import CustomMaskRCNNTileOptimized
from .custom_single_stage_detector import CustomSingleStageDetector
from .custom_two_stage_detector import CustomTwoStageDetector
from .custom_vfnet_detector import CustomVFNet
from .custom_yolox_detector import CustomYOLOX
from .l2sp_detector_mixin import L2SPDetectorMixin
from .sam_detector_mixin import SAMDetectorMixin
from .unbiased_teacher import UnbiasedTeacher

__all__ = [
    "CustomATSS",
    "CustomMaskRCNN",
    "CustomSingleStageDetector",
    "CustomTwoStageDetector",
    "CustomVFNet",
    "CustomYOLOX",
    "L2SPDetectorMixin",
    "SAMDetectorMixin",
    "UnbiasedTeacher",
    "CustomMaskRCNNTileOptimized",
]



"""Collections of hooks for common OTX algorithms."""

# Copyright (C) 2021-2023 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

from mmcv.runner.hooks import HOOKS, Hook

from otx.algorithms.common.utils.logger import get_logger

logger = get_logger()


@HOOKS.register_module()
class ForceTrainModeHook(Hook):
    """Force train mode for model.

    This is a workaround of a bug in EvalHook from MMCV.
    If a model evaluation is enabled before training by setting 'start=0' in EvalHook,
    EvalHook does not put a model in a training mode again after evaluation.

    This simple hook forces to put a model in a training mode before every train epoch
    with the lowest priority.
    """

    def before_train_epoch(self, runner):
        """Make sure to put a model in a training mode before train epoch."""
        runner.model.train()



"""MobileNet-V3-large-1 for self-supervised learning config."""

# pylint: disable=invalid-name

_base_ = ["../../../../../recipes/stages/classification/selfsl.yaml", "../../base/models/mobilenet_v3.py"]

model = dict(
    type="BYOL",
    task="classification",
    backbone=dict(mode="large"),
    base_momentum=0.996,
    neck=dict(type="SelfSLMLP", in_channels=960, hid_channels=4096, out_channels=256, with_avg_pool=True),
    head=dict(
        _delete_=True,
        type="ConstrastiveHead",
        predictor=dict(type="SelfSLMLP", in_channels=256, hid_channels=4096, out_channels=256, with_avg_pool=False),
        in_channels=960,
        hid_channels=1280,
    ),
)

load_from = None

resume_from = None

fp16 = None



"""Detection models dynamic deploy config."""

_base_ = ["./base_detection_static.py"]

ir_config = dict(
    dynamic_axes={
        "image": {
            0: "batch",
            1: "channel",
            2: "height",
            3: "width",
        },
        "boxes": {
            0: "batch",
            1: "num_dets",
        },
        "labels": {
            0: "batch",
            1: "num_dets",
        },
    },
)



"""Unit test for otx.algorithms.common.adapters.mmcv.hooks.checkpoint_hook."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from mmcv.utils import Config

from otx.algorithms.common.adapters.mmcv.hooks.checkpoint_hook import (
    CheckpointHookWithValResults,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockRunner:
    class _MockModel:
        def __init__(self, string):
            self.name = string

        def buffers(self):
            return None

    class _MockLogger:
        def info(self, string):
            print(string)

    def __init__(self):
        self.model = self._MockModel("model")
        self.ema_model = self._MockModel("ema_model")
        self.meta = {}
        self.epoch = 1
        self.iter = 1
        self.save_ckpt = True
        self.logger = self._MockLogger()
        self.save_ema_model = True

    def save_checkpoint(self, *args, **kwrags):
        pass


class TestCheckpointHookWithValResults:
    """Test class for CheckpointHookWithValResults."""

    @e2e_pytest_unit
    def test_before_run(self) -> None:
        """Test before_run function."""

        hook = CheckpointHookWithValResults()
        mock_runner = Config({"work_dir": "./temp_dir/"})
        hook.before_run(mock_runner)
        assert hook.out_dir == "./temp_dir/"

    @e2e_pytest_unit
    def test_after_train_epoch(self, mocker) -> None:
        """Test after_train_epoch function."""

        mocker.patch.object(CheckpointHookWithValResults, "every_n_epochs", return_value=True)
        mocker.patch("otx.algorithms.common.adapters.mmcv.hooks.checkpoint_hook.allreduce_params", return_value=True)
        hook = CheckpointHookWithValResults(sync_buffer=True, out_dir="./tmp_dir/")
        runner = MockRunner()
        hook.after_train_epoch(runner)

        assert runner.model.name == "model"
        assert runner.save_ema_model is False



"""Module for data caching."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from .mem_cache_handler import MemCacheHandlerError, MemCacheHandlerSingleton
from .mem_cache_hook import MemCacheHook
from .storage_cache import init_arrow_cache

__all__ = ["MemCacheHandlerSingleton", "MemCacheHook", "MemCacheHandlerError", "init_arrow_cache"]



"""EfficientNet-B0 for multi-label config."""

# pylint: disable=invalid-name

_base_ = ["../../../../recipes/stages/classification/multilabel/incremental.yaml", "../base/models/efficientnet.py"]

model = dict(
    type="SAMImageClassifier",
    task="classification",
    backbone=dict(
        version="b0",
    ),
    head=dict(
        type="CustomMultiLabelLinearClsHead",
        normalized=True,
        scale=7.0,
        loss=dict(type="AsymmetricAngularLossWithIgnore", gamma_pos=0.0, gamma_neg=1.0, reduction="sum"),
    ),
)

fp16 = dict(loss_scale=512.0)



import copy
import os

import pytest
import tempfile
from mmcv.utils import ConfigDict

from otx.algorithms.common.adapters.mmcv.utils.config_utils import MPAConfig
from otx.algorithms.classification.adapters.mmcls.configurer import (
    ClassificationConfigurer,
    IncrClassificationConfigurer,
    SemiSLClassificationConfigurer,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.algorithms.classification.test_helper import DEFAULT_CLS_TEMPLATE_DIR


class TestClassificationConfigurer:
    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        self.configurer = ClassificationConfigurer()
        self.model_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_CLS_TEMPLATE_DIR, "model.py"))
        self.data_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_CLS_TEMPLATE_DIR, "data_pipeline.py"))

    @e2e_pytest_unit
    def test_configure(self, mocker):
        mock_cfg_base = mocker.patch.object(ClassificationConfigurer, "configure_base")
        mock_cfg_device = mocker.patch.object(ClassificationConfigurer, "configure_device")
        mock_cfg_ckpt = mocker.patch.object(ClassificationConfigurer, "configure_ckpt")
        mock_cfg_model = mocker.patch.object(ClassificationConfigurer, "configure_model")
        mock_cfg_data = mocker.patch.object(ClassificationConfigurer, "configure_data")
        mock_cfg_task = mocker.patch.object(ClassificationConfigurer, "configure_task")
        mock_cfg_hook = mocker.patch.object(ClassificationConfigurer, "configure_hook")
        mock_cfg_gpu = mocker.patch.object(ClassificationConfigurer, "configure_samples_per_gpu")
        mock_cfg_fp16_optimizer = mocker.patch.object(ClassificationConfigurer, "configure_fp16_optimizer")
        mock_cfg_compat_cfg = mocker.patch.object(ClassificationConfigurer, "configure_compat_cfg")

        model_cfg = copy.deepcopy(self.model_cfg)
        data_cfg = copy.deepcopy(self.data_cfg)
        returned_value = self.configurer.configure(model_cfg, "", data_cfg, True)
        mock_cfg_base.assert_called_once_with(model_cfg, data_cfg, None, None)
        mock_cfg_device.assert_called_once_with(model_cfg, True)
        mock_cfg_ckpt.assert_called_once_with(model_cfg, "")
        mock_cfg_model.assert_called_once_with(model_cfg, None)
        mock_cfg_data.assert_called_once_with(model_cfg, True, data_cfg)
        mock_cfg_task.assert_called_once_with(model_cfg, True)
        mock_cfg_hook.assert_called_once_with(model_cfg)
        mock_cfg_gpu.assert_called_once_with(model_cfg, "train")
        mock_cfg_fp16_optimizer.assert_called_once_with(model_cfg)
        mock_cfg_compat_cfg.assert_called_once_with(model_cfg)
        assert returned_value == model_cfg

    @e2e_pytest_unit
    def test_configure_base(self, mocker):
        mocker.patch(
            "otx.algorithms.classification.adapters.mmcls.configurer.align_data_config_with_recipe",
            return_value=True,
        )
        mocker.patch(
            "otx.algorithms.classification.adapters.mmcls.configurer.patch_datasets",
            return_value=True,
        )
        mocker.patch(
            "otx.algorithms.classification.adapters.mmcls.configurer.patch_persistent_workers",
            return_value=True,
        )

        model_cfg = copy.deepcopy(self.model_cfg)
        data_cfg = copy.deepcopy(self.data_cfg._cfg_dict)
        self.configurer.configure_base(model_cfg, data_cfg, [], [])

    @e2e_pytest_unit
    def test_configure_device(self, mocker):
        mocker.patch(
            "torch.distributed.is_initialized",
            return_value=True,
        )
        mocker.patch("os.environ", return_value={"LOCAL_RANK": 2})
        config = copy.deepcopy(self.model_cfg)
        self.configurer.configure_device(config, True)
        assert config.distributed is True

        mocker.patch(
            "torch.distributed.is_initialized",
            return_value=False,
        )
        mocker.patch(
            "torch.cuda.is_available",
            return_value=False,
        )
        config = copy.deepcopy(self.model_cfg)
        self.configurer.configure_device(config, True)
        assert config.distributed is False
        assert config.device == "cpu"

        mocker.patch(
            "torch.distributed.is_initialized",
            return_value=False,
        )
        mocker.patch(
            "torch.cuda.is_available",
            return_value=True,
        )
        config = copy.deepcopy(self.model_cfg)
        self.configurer.configure_device(config, True)
        assert config.distributed is False
        assert config.device == "cuda"

    @e2e_pytest_unit
    def test_configure_model(self):
        ir_options = {"ir_model_path": {"ir_weight_path": "", "ir_weight_init": ""}}
        self.model_cfg.model.head.in_channels = -1
        self.configurer.configure_model(self.model_cfg, ir_options)
        assert self.model_cfg.model_task
        assert self.model_cfg.model.head.in_channels == 960

    @e2e_pytest_unit
    def test_configure_model_not_classification_task(self):
        ir_options = {"ir_model_path": {"ir_weight_path": "", "ir_weight_init": ""}}
        configure_cfg = copy.deepcopy(self.model_cfg)
        configure_cfg.model.task = "detection"
        with pytest.raises(ValueError):
            self.configurer.configure_model(configure_cfg, ir_options)

    @e2e_pytest_unit
    def test_configure_ckpt(self, mocker):
        model_cfg = copy.deepcopy(self.model_cfg)
        model_cfg.resume = True

        mocker.patch(
            "otx.algorithms.classification.adapters.mmcls.configurer.CheckpointLoader.load_checkpoint",
            return_value={"model": None},
        )
        with tempfile.TemporaryDirectory() as tempdir:
            self.configurer.configure_ckpt(model_cfg, os.path.join(tempdir, "dummy.pth"))

    @e2e_pytest_unit
    def test_configure_data(self, mocker):
        data_cfg = copy.deepcopy(self.data_cfg)
        data_cfg.data.pipeline_options = dict(
            MinIouRandomCrop=dict(min_crop_size=0.1),
            Resize=dict(
                img_scale=[(1344, 480), (1344, 960)],
                multiscale_mode="range",
            ),
            Normalize=dict(),
            MultiScaleFlipAug=dict(
                img_scale=(1344, 800),
                flip=False,
                transforms=[
                    dict(type="Resize", keep_ratio=False),
                    dict(type="Normalize"),
                    dict(type="Pad", size_divisor=32),
                    dict(type="ImageToTensor", keys=["img"]),
                    dict(type="Collect", keys=["img"]),
                ],
            ),
        )
        self.configurer.configure_data(self.model_cfg, True, data_cfg)
        assert self.model_cfg.data
        assert self.model_cfg.data.train
        assert self.model_cfg.data.val

    @e2e_pytest_unit
    def test_configure_task(self, mocker):
        model_cfg = copy.deepcopy(self.model_cfg)
        model_cfg.update(self.data_cfg)
        model_cfg.task_adapt = {"type": "mpa", "op": "REPLACE", "use_mpa_anchor": True}
        self.configurer.configure_task(model_cfg, True)

        self.configurer.model_classes = []
        self.configurer.data_classes = ["red", "green"]
        self.configurer.configure_task(model_cfg, True)

    @e2e_pytest_unit
    def test_configure_hook(self):
        model_cfg = copy.deepcopy(self.model_cfg)
        model_cfg.custom_hooks = [{"type": "LazyEarlyStoppingHook", "start": 3}]
        model_cfg.custom_hook_options = {"LazyEarlyStoppingHook": {"start": 5}, "LoggerReplaceHook": {"_delete_": True}}
        self.configurer.configure_hook(model_cfg)
        assert model_cfg.custom_hooks[0]["start"] == 5

    @e2e_pytest_unit
    def test_configure_samples_per_gpu(self):
        model_cfg = copy.deepcopy(self.model_cfg)
        model_cfg.update(self.data_cfg)
        model_cfg.data.train.otx_dataset = range(1)
        self.configurer.configure_samples_per_gpu(model_cfg, "train")
        assert model_cfg.data.train_dataloader == {"samples_per_gpu": 1, "drop_last": True}

    @e2e_pytest_unit
    def test_configure_fp16_optimizer(self):
        model_cfg = copy.deepcopy(self.model_cfg)
        model_cfg.fp16 = {}
        model_cfg.optimizer_config.type = "OptimizerHook"
        self.configurer.configure_fp16_optimizer(model_cfg)
        assert model_cfg.optimizer_config.type == "Fp16OptimizerHook"

        model_cfg.fp16 = {}
        model_cfg.optimizer_config.type = "SAMOptimizerHook"
        self.configurer.configure_fp16_optimizer(model_cfg)
        assert model_cfg.optimizer_config.type == "Fp16SAMOptimizerHook"

        model_cfg.fp16 = {}
        model_cfg.optimizer_config.type = "DummyOptimizerHook"
        self.configurer.configure_fp16_optimizer(model_cfg)
        assert model_cfg.optimizer_config.type == "DummyOptimizerHook"

    @e2e_pytest_unit
    def test_configure_compat_cfg(self):
        model_cfg = copy.deepcopy(self.model_cfg)
        model_cfg.update(self.data_cfg)
        model_cfg.data.train_dataloader = {}
        model_cfg.data.val_dataloader = {}
        model_cfg.data.test_dataloader = {}
        self.configurer.configure_compat_cfg(model_cfg)

    @e2e_pytest_unit
    def test_get_data_cfg(self):
        config = copy.deepcopy(self.model_cfg)
        config.update(self.data_cfg)
        config.data.train.dataset = ConfigDict({"dataset": [1, 2, 3]})
        assert [1, 2, 3] == self.configurer.get_data_cfg(config, "train")


class TestIncrClassificationConfigurer:
    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        self.configurer = IncrClassificationConfigurer()
        self.model_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_CLS_TEMPLATE_DIR, "model.py"))
        self.data_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_CLS_TEMPLATE_DIR, "data_pipeline.py"))

    def test_configure_task(self, mocker):
        mocker.patch.object(ClassificationConfigurer, "configure_task")
        self.model_cfg.update(self.data_cfg)
        self.model_cfg.task_adapt = {}
        self.configurer.task_adapt_type = "mpa"
        self.configurer.configure_task(self.model_cfg, True)
        assert self.model_cfg.custom_hooks[0].type == "TaskAdaptHook"
        assert self.model_cfg.custom_hooks[0].sampler_flag is False


class TestSemiSLClassificationConfigurer:
    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        self.configurer = SemiSLClassificationConfigurer()
        self.model_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_CLS_TEMPLATE_DIR, "model.py"))
        self.data_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_CLS_TEMPLATE_DIR, "data_pipeline.py"))

    def test_configure_data(self, mocker):
        mocker.patch.object(ClassificationConfigurer, "configure_data")
        mocker.patch("mmdet.datasets.build_dataset", return_value=[])
        mocker.patch("otx.algorithms.classification.adapters.mmcls.configurer.build_dataloader", return_value=[])
        self.model_cfg.update(self.data_cfg)
        self.model_cfg.data.unlabeled = ConfigDict({"type": "OTXDataset", "otx_dataset": range(10)})
        self.model_cfg.model_task = "detection"
        self.model_cfg.distributed = False
        self.configurer.configure_data(self.model_cfg, True, self.data_cfg)

    def test_configure_task(self):
        self.model_cfg.update(self.data_cfg)
        self.model_cfg.task_adapt = {"type": "mpa", "op": "REPLACE", "use_mpa_anchor": True}
        self.configurer.configure_task(self.model_cfg, True)

        self.model_cfg.task_adapt = {"type": "not_mpa", "op": "REPLACE", "use_mpa_anchor": True}
        self.configurer.configure_task(self.model_cfg, True)



"""OTX Algorithms - Action Recognition."""

# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0

import os

os.environ["FEATURE_FLAGS_OTX_ACTION_TASKS"] = "1"

MMACTION_AVAILABLE = True

try:
    import mmaction  # noqa: F401
except ImportError:
    MMACTION_AVAILABLE = False



# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import pytest
import numpy as np

from otx.api.entities.annotation import (
    Annotation,
    AnnotationSceneEntity,
    AnnotationSceneKind,
)
from otx.api.entities.dataset_item import DatasetItemEntityWithID
from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.id import ID
from otx.api.entities.image import Image
from otx.api.entities.label import Domain, LabelEntity
from otx.api.entities.scored_label import ScoredLabel
from otx.api.entities.shapes.rectangle import Rectangle


@pytest.fixture()
def fxt_multi_class_cls_dataset_entity() -> DatasetEntity:
    labels = ["car", "dog", "cat"]
    items = []
    for label in labels:
        image = Image(data=np.random.randint(low=0, high=255, size=(8, 8, 3)))
        annotation = Annotation(
            shape=Rectangle.generate_full_box(),
            labels=[ScoredLabel(LabelEntity(name=label, domain=Domain.CLASSIFICATION, id=ID(label)))],
        )
        annotation_scene = AnnotationSceneEntity(annotations=[annotation], kind=AnnotationSceneKind.ANNOTATION)
        items += [DatasetItemEntityWithID(media=image, annotation_scene=annotation_scene, id_=ID(label))]

    dataset = DatasetEntity(items=items)
    return dataset



from pathlib import Path
from pprint import pformat

from otx.api.entities.model_template import ModelTemplate
from otx.cli.utils.report import (
    data_config_to_str,
    env_info_to_str,
    get_otx_report,
    sub_title_to_str,
    task_config_to_str,
    template_info_to_str,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockModelTemplate(ModelTemplate):
    """Mock class for ModelTemplate."""

    def __init__(self):
        self.test1 = "abc"
        self.test2 = "cba"


@e2e_pytest_unit
def test_sub_title_to_str():
    expected = "-" * 60 + "\n\n" + "test" + "\n\n" + "-" * 60 + "\n"
    result = sub_title_to_str("test")
    assert expected == result


@e2e_pytest_unit
def test_env_info_to_str(mocker):
    expected = "\tOTX: 1.2\n"
    mocker.patch("mmcv.utils.env.collect_env", return_value={"OTX": "1.2"})
    result = env_info_to_str()
    assert expected == result


@e2e_pytest_unit
def test_template_info_to_str():
    mock_template = MockModelTemplate()
    expected = f"\ttest1: {pformat('abc')}\n" + f"\ttest2: {pformat('cba')}\n"
    result = template_info_to_str(mock_template)
    assert expected == result


@e2e_pytest_unit
def test_data_config_to_str():
    data_config = {
        "train_subset": {"data-roots": "aaa"},
        "val_subset": {"data-roots": "aaa"},
    }
    expected = "train_subset:\n\tdata-roots: aaa\nval_subset:\n\tdata-roots: aaa\n"
    result = data_config_to_str(data_config)
    assert expected == result


@e2e_pytest_unit
def test_task_config_to_str():
    task_config = {"a": "b", "b": "c"}
    expected = "a: 'b'\nb: 'c'\n"
    result = task_config_to_str(task_config)
    assert expected == result


@e2e_pytest_unit
def test_get_otx_report(tmp_dir):
    report_path = Path(tmp_dir) / "report.log"
    model_template = MockModelTemplate()
    task_config = {"a": "b", "b": "c"}
    data_config = {
        "train_subset": {"data-roots": "aaa"},
        "val_subset": {"data-roots": "aaa"},
    }
    get_otx_report(
        model_template=model_template,
        task_config=task_config,
        data_config=data_config,
        results={"time": "0:01"},
        output_path=str(report_path),
    )
    assert report_path.exists()



"""This module tests classes related to OptimizationParameters"""

# Copyright (C) 2020-2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import dataclasses

import pytest

from otx.api.entities.optimization_parameters import OptimizationParameters
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestOptimizationParameters:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_optimization_parameters_members(self):
        """
        <b>Description:</b>
        To test OptimizationParameters dataclass members

        <b>Input data:</b>
        Initialized instance of OptimizationParameters class

        <b>Expected results:</b>

        <b>Steps</b>
        1. Create OptimizationParameters
        2. Check members
        """
        opt_params = OptimizationParameters()

        assert dataclasses.is_dataclass(opt_params)
        assert len(dataclasses.fields(opt_params)) == 3
        assert dataclasses.fields(opt_params)[0].name == "resume"
        assert dataclasses.fields(opt_params)[1].name == "update_progress"
        assert dataclasses.fields(opt_params)[2].name == "save_model"
        assert type(opt_params.resume) is bool
        assert callable(opt_params.update_progress)
        assert callable(opt_params.save_model)
        with pytest.raises(AttributeError):
            str(opt_params.WRONG)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_optimization_parameters_update_member(self):
        """
        <b>Description:</b>
        To test OptimizationParameters dataclass members update

        <b>Input data:</b>
        Initialized instance of OptimizationParameters class

        <b>Expected results:</b>

        <b>Steps</b>
        1. Initiate OptimizationParameters instance
        2. Check members update
        """
        opt_params = OptimizationParameters(False)
        assert opt_params.resume is False
        assert (
            opt_params.update_progress(0)
            is opt_params.update_progress(50.5)
            is opt_params.update_progress(100)
            is opt_params.update_progress(0, 0.3)
            is opt_params.update_progress(50.5, 1.4)
            is opt_params.update_progress(100, -6.1)
            is None
        )
        assert opt_params.save_model() is None

        opt_params = OptimizationParameters(True)
        assert opt_params.resume is True
        assert (
            opt_params.update_progress(0)
            is opt_params.update_progress(50.5)
            is opt_params.update_progress(100)
            is opt_params.update_progress(0, 0.3)
            is opt_params.update_progress(50.5, 1.4)
            is opt_params.update_progress(100, -6.1)
            is None
        )
        assert opt_params.save_model() is None



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#


import pickle  # nosec

import pytest
from bson import ObjectId

from otx.api.entities.id import ID
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestPickle:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_pickle_id(self):
        """
        <b>Description:</b>
        Check ID can be correctly pickled and unpickled

        <b>Input data:</b>
        ID

        <b>Expected results:</b>
        Test passes if the ID is correctly unpickled

        <b>Steps</b>
        1. Create ID
        2. Pickle and unpickle ID
        3. Check ID against unpickled ID
        """
        original_id = ID(ObjectId())
        pickled_id = pickle.dumps(original_id)
        unpickled_id = pickle.loads(pickled_id)  # nosec
        assert id(original_id) != id(pickled_id), "Expected two different memory instanced"
        assert original_id == unpickled_id, "Expected content of entities to be equal"



"""Unit Test for otx.algorithms.action.adapters.mmaction.task."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os
from copy import deepcopy
from typing import Any, Dict

import numpy as np
import pytest
import torch
from mmaction.models.backbones.x3d import X3D
from mmaction.models.recognizers.recognizer3d import Recognizer3D
from mmcv.runner import CheckpointLoader
from mmcv.utils import Config
from torch import nn

from otx.algorithms.action.configs.base.configuration import ActionConfig
from otx.algorithms.action.adapters.mmaction.task import MMActionTask
from otx.algorithms.common.adapters.mmcv.utils.config_utils import MPAConfig
from otx.api.configuration import ConfigurableParameters
from otx.api.configuration.helper import create
from otx.api.entities.dataset_item import DatasetItemEntity
from otx.api.entities.datasets import DatasetEntity
from otx.api.entities.inference_parameters import InferenceParameters
from otx.api.entities.label import Domain
from otx.api.entities.label_schema import LabelGroup, LabelGroupType, LabelSchemaEntity
from otx.api.entities.model import (
    ModelConfiguration,
    ModelEntity,
    ModelFormat,
    ModelOptimizationType,
    ModelPrecision,
)
from otx.api.entities.model_template import InstantiationType, parse_model_template, TaskFamily, TaskType
from otx.api.entities.resultset import ResultSetEntity
from otx.api.usecases.tasks.interfaces.export_interface import ExportType
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.algorithms.action.test_helpers import (
    init_environment,
    MockModelTemplate,
    generate_action_cls_otx_dataset,
    generate_action_det_otx_dataset,
    generate_labels,
    return_inputs,
)

DEFAULT_ACTION_CLS_DIR = os.path.join("otx/algorithms/action/configs/classification", "x3d")
DEFAULT_ACTION_DET_DIR = os.path.join("otx/algorithms/action/configs/detection", "x3d_fast_rcnn")


class MockModule(nn.Module):
    """Mock class for nn.Module."""

    def forward(self, inputs: Any):
        return inputs


class MockModel(nn.Module):
    """Mock class for pytorch model."""

    def __init__(self, task_type):
        super().__init__()
        self.module = MockModule()
        self.module.backbone = MockModule()
        self.backbone = MockModule()
        self.task_type = task_type

    def forward(self, return_loss: bool, imgs: DatasetItemEntity):
        forward_hooks = list(self.module.backbone._forward_hooks.values())
        for hook in forward_hooks:
            hook(1, 2, 3)
        if self.task_type == "cls":
            return np.array([[0, 0, 1]])
        return [[np.array([[0, 0, 1, 1, 0.1]]), np.array([[0, 0, 1, 1, 0.2]]), np.array([[0, 0, 1, 1, 0.7]])]]

    @staticmethod
    def named_parameters():
        return {"name": torch.Tensor([0.5])}.items()


class MockDataset(DatasetEntity):
    """Mock class for mm_dataset."""

    def __init__(self, dataset: DatasetEntity, task_type: str):
        self.dataset = dataset
        self.task_type = task_type
        self.CLASSES = ["1", "2", "3"]

    def evaluate(self, prediction, *args, **kwargs):
        if self.task_type == "cls":
            return {"mean_class_accuracy": 1.0}
        else:
            return {"mAP@0.5IOU": 1.0}


class MockDataLoader:
    """Mock class for data loader."""

    def __init__(self, dataset: DatasetEntity):
        self.dataset = dataset
        self.iter = iter(self.dataset)

    def __len__(self) -> int:
        return len(self.dataset)

    def __next__(self) -> Dict[str, DatasetItemEntity]:
        return {"imgs": next(self.iter)}

    def __iter__(self):
        return self


class MockExporter:
    """Mock class for Exporter."""

    def __init__(self, task):
        self.work_dir = task._output_path

    def export(self):
        dummy_data = np.ndarray((1, 1, 1))
        with open(os.path.join(self.work_dir, "openvino.bin"), "wb") as f:
            f.write(dummy_data)
        with open(os.path.join(self.work_dir, "openvino.xml"), "wb") as f:
            f.write(dummy_data)
        with open(os.path.join(self.work_dir, "model.onnx"), "wb") as f:
            f.write(dummy_data)


class TestMMActionTask:
    """Test class for MMActionTask.

    Details are explained in each test function.
    """

    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        self.video_len = 3
        self.frame_len = 3

        cls_labels = generate_labels(3, Domain.ACTION_CLASSIFICATION)
        self.cls_label_schema = LabelSchemaEntity()
        cls_label_group = LabelGroup(
            name="labels",
            labels=cls_labels,
            group_type=LabelGroupType.EXCLUSIVE,
        )
        self.cls_label_schema.add_group(cls_label_group)
        self.cls_dataset = generate_action_cls_otx_dataset(self.video_len, self.frame_len, cls_labels)

        cls_model_template = parse_model_template(os.path.join(DEFAULT_ACTION_CLS_DIR, "template.yaml"))
        cls_hyper_parameters = create(cls_model_template.hyper_parameters.data)
        cls_task_env = init_environment(cls_hyper_parameters, cls_model_template, self.cls_label_schema)
        self.cls_task = MMActionTask(task_environment=cls_task_env)

        det_labels = generate_labels(3, Domain.ACTION_DETECTION)
        self.det_label_schema = LabelSchemaEntity()
        det_label_group = LabelGroup(
            name="labels",
            labels=det_labels,
            group_type=LabelGroupType.EXCLUSIVE,
        )
        self.det_label_schema.add_group(det_label_group)
        self.det_dataset = generate_action_det_otx_dataset(self.video_len, self.frame_len, det_labels)[0]

        det_model_template = parse_model_template(os.path.join(DEFAULT_ACTION_DET_DIR, "template.yaml"))
        det_hyper_parameters = create(det_model_template.hyper_parameters.data)
        det_task_env = init_environment(det_hyper_parameters, det_model_template, self.det_label_schema)
        self.det_task = MMActionTask(task_environment=det_task_env)

    @e2e_pytest_unit
    def test_build_model(self, mocker) -> None:
        """Test build_model function."""
        _weight = torch.randn([24, 3, 1, 3, 3])
        mocker.patch.object(
            CheckpointLoader,
            "load_checkpoint",
            return_value={"model": {"state_dict": {"backbone.conv1_s.conv.weight": _weight}}},
        )
        _mock_recipe_cfg = MPAConfig.fromfile(os.path.join(DEFAULT_ACTION_CLS_DIR, "model.py"))
        model = self.cls_task.build_model(_mock_recipe_cfg, True)
        assert isinstance(model, Recognizer3D)
        assert isinstance(model.backbone, X3D)
        assert torch.all(model.state_dict()["backbone.conv1_s.conv.weight"] == _weight)

    @e2e_pytest_unit
    def test_train(self, mocker) -> None:
        """Test train function."""
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataset",
            return_value=MockDataset(self.cls_dataset, "cls"),
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataloader",
            return_value=MockDataLoader(self.cls_dataset),
        )
        mocker.patch.object(MMActionTask, "build_model", return_value=MockModel("cls"))
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.patch_data_pipeline",
            return_value=True,
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_data_parallel",
            return_value=MockModel("cls"),
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.train_model",
            return_value=True,
        )
        mocker.patch("torch.load", return_value={"state_dict": np.ndarray([1, 1, 1])})

        _config = ModelConfiguration(ActionConfig(), self.cls_label_schema)
        output_model = ModelEntity(self.cls_dataset, _config)
        self.cls_task.train(self.cls_dataset, output_model)
        output_model.performance == 1.0

        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataset",
            return_value=MockDataset(self.det_dataset, "det"),
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataloader",
            return_value=MockDataLoader(self.det_dataset),
        )
        mocker.patch.object(MMActionTask, "build_model", return_value=MockModel("det"))
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.patch_data_pipeline",
            return_value=True,
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_data_parallel",
            return_value=MockModel("det"),
        )
        _config = ModelConfiguration(ActionConfig(), self.det_label_schema)
        output_model = ModelEntity(self.det_dataset, _config)
        self.det_task.train(self.det_dataset, output_model)
        output_model.performance == 1.0

    @e2e_pytest_unit
    def test_infer(self, mocker) -> None:
        """Test infer function.

        <Steps>
            1. Create mock model for action classification
            2. Create mock recipe for action classification
            3. Run infer funciton
            4. Check whether inference results are added to output
            5. Do 1 - 4 for action detection
        """
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataset",
            return_value=MockDataset(self.cls_dataset, "cls"),
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataloader",
            return_value=MockDataLoader(self.cls_dataset),
        )
        mocker.patch.object(MMActionTask, "build_model", return_value=MockModel("cls"))
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.patch_data_pipeline",
            return_value=True,
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_data_parallel",
            return_value=MockModel("cls"),
        )

        inference_parameters = InferenceParameters(is_evaluation=True)
        outputs = self.cls_task.infer(self.cls_dataset, inference_parameters)
        for output in outputs:
            assert len(output.get_annotations()[0].get_labels()) == 2

        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataset",
            return_value=MockDataset(self.det_dataset, "det"),
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_dataloader",
            return_value=MockDataLoader(self.det_dataset),
        )
        mocker.patch.object(MMActionTask, "build_model", return_value=MockModel("det"))
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.patch_data_pipeline",
            return_value=True,
        )
        mocker.patch(
            "otx.algorithms.action.adapters.mmaction.task.build_data_parallel",
            return_value=MockModel("det"),
        )
        inference_parameters = InferenceParameters(is_evaluation=True)
        outputs = self.det_task.infer(self.det_dataset, inference_parameters)
        for output in outputs:
            assert len(output.get_annotations()) == 2

    @e2e_pytest_unit
    def test_evaluate(self) -> None:
        """Test evaluate function."""
        _config = ModelConfiguration(ActionConfig(), self.cls_label_schema)
        _model = ModelEntity(self.cls_dataset, _config)
        resultset = ResultSetEntity(_model, self.cls_dataset, self.cls_dataset)
        self.cls_task.evaluate(resultset)
        assert resultset.performance.score.value == 1.0

    @e2e_pytest_unit
    def test_evaluate_with_empty_annot(self) -> None:
        """Test evaluate function with empty_annot."""
        _config = ModelConfiguration(ActionConfig(), self.cls_label_schema)
        _model = ModelEntity(self.cls_dataset, _config)
        resultset = ResultSetEntity(_model, self.cls_dataset, self.cls_dataset.with_empty_annotations())
        self.cls_task.evaluate(resultset)
        assert resultset.performance.score.value == 0.0

    @e2e_pytest_unit
    def test_evaluate_det(self) -> None:
        """Test evaluate function for action detection."""
        _config = ModelConfiguration(ActionConfig(), self.det_label_schema)
        _model = ModelEntity(self.det_dataset, _config)
        resultset = ResultSetEntity(_model, self.det_dataset, self.det_dataset)
        self.det_task.evaluate(resultset)
        assert resultset.performance.score.value == 0.0

    @pytest.mark.parametrize("precision", [ModelPrecision.FP16, ModelPrecision.FP32])
    @e2e_pytest_unit
    def test_export(self, mocker, precision: ModelPrecision) -> None:
        """Test export function.

        <Steps>
            1. Create model entity
            2. Run export function
            3. Check output model attributes
        """
        _config = ModelConfiguration(ActionConfig(), self.cls_label_schema)
        _model = ModelEntity(self.cls_dataset, _config)
        mocker.patch("otx.algorithms.action.adapters.mmaction.task.Exporter", return_value=MockExporter(self.cls_task))
        mocker.patch("torch.load", return_value={})
        mocker.patch("torch.nn.Module.load_state_dict", return_value=True)

        self.cls_task.export(ExportType.OPENVINO, _model, precision, False)

        assert _model.model_format == ModelFormat.OPENVINO
        assert _model.optimization_type == ModelOptimizationType.MO
        assert _model.precision[0] == precision
        assert _model.get_data("openvino.bin") is not None
        assert _model.get_data("openvino.xml") is not None
        assert _model.get_data("confidence_threshold") is not None
        assert _model.precision == self.cls_task._precision
        assert _model.optimization_methods == self.cls_task._optimization_methods
        assert _model.get_data("label_schema.json") is not None



import argparse
from pathlib import Path

import pytest

from otx.cli.tools import deploy as target_package
from otx.cli.tools.deploy import get_args, main
from tests.test_suite.e2e_test_system import e2e_pytest_unit


@e2e_pytest_unit
def test_get_args(mocker):
    mocker.patch("sys.argv", ["otx", "--load-weights", "load_weights", "--output", "output"])
    mocker.patch.object(
        target_package, "get_parser_and_hprams_data", return_value=[argparse.ArgumentParser(), "fake", "fake"]
    )

    parsed_args = get_args()

    assert parsed_args.load_weights == "load_weights"
    assert parsed_args.output == "output"


@pytest.fixture
def mock_args(mocker, tmp_dir):
    mock_args = mocker.MagicMock()
    mock_args.load_weights = "fake.bin"
    mock_args.output = tmp_dir

    def mock_contains(self, val):
        return val in self.__dict__

    mock_args.__contains__ = mock_contains
    mock_get_args = mocker.patch("otx.cli.tools.deploy.get_args")
    mock_get_args.return_value = mock_args

    return mock_args


@pytest.fixture
def mock_task(mocker):
    mock_task_class = mocker.MagicMock()
    mock_task = mocker.MagicMock()
    mock_task_class.return_value = mock_task
    mocker.patch.object(target_package, "get_impl_class", return_value=mock_task_class)

    return mock_task


@pytest.fixture
def mock_config_manager(mocker):
    mock_config_manager = mocker.patch.object(target_package, "ConfigManager")
    mock_template = mocker.MagicMock()
    mock_template.name = "fake_name"
    mock_config_manager.return_value.template = mock_template

    return mock_config_manager


@e2e_pytest_unit
def test_main(mocker, mock_args, mock_task, mock_config_manager, tmp_dir):
    # prepare
    mocker.patch.object(target_package, "TaskEnvironment")
    mocker.patch.object(target_package, "create")
    mocker.patch.object(target_package, "read_model")
    mocker.patch.object(target_package, "read_label_schema")
    mock_deployed_model = mocker.MagicMock()
    mock_deployed_model.exportable_code = b"exportable_code"
    mocker.patch.object(target_package, "ModelEntity", return_value=mock_deployed_model)

    # run
    ret = main()

    # check
    assert ret["retcode"] == 0
    assert ret["template"] == "fake_name"
    mock_task.deploy.assert_called_once_with(mock_deployed_model)
    with (Path(tmp_dir) / "openvino.zip").open() as f:
        val = f.readline()
        assert val == "exportable_code"


@e2e_pytest_unit
def test_main_wrong_workspace(mock_args, mock_config_manager):
    mock_args.load_weights = ""
    mock_config_manager.check_workspace.return_value = True

    with pytest.raises(RuntimeError):
        main()


@e2e_pytest_unit
@pytest.mark.parametrize("load_weights", ["fake.jpg", "fake.png"])
def test_main_wrong_laod_weight(mocker, load_weights):
    # prepare
    mock_agrs = mocker.MagicMock()
    mock_agrs.load_weights = load_weights
    mocker.patch.object(target_package, "get_args", return_value=mock_agrs)
    mocker.patch.object(target_package, "ConfigManager")

    # run
    with pytest.raises(RuntimeError):
        main()



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from types import FunctionType

import pytest
from attr import fields

from otx.api.configuration import ConfigurableParameters
from otx.api.configuration.elements.parameter_group import ParameterGroup
from otx.api.configuration.elements.utils import (
    _convert_enum_selectable_value,
    _validate_and_convert_float,
    attr_enum_to_str_serializer,
    attr_strict_float_converter,
    attr_strict_float_on_setattr,
    attr_strict_int_validator,
    construct_attr_enum_selectable_converter,
    construct_attr_enum_selectable_onsetattr,
    construct_attr_selectable_validator,
    construct_attr_value_validator,
    convert_string_to_id,
)
from otx.api.configuration.enums.config_element_type import ElementCategory
from otx.api.entities.id import ID
from tests.unit.api.configuration.dummy_config import SomeEnumSelectable
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestUtilsFunctions:
    parameter_group = ParameterGroup(header="test header")
    attribute = fields(ConfigurableParameters)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_utils_attr_enum_to_str_serializer(self):
        """
        <b>Description:</b>
        Check "attr_enum_to_str_serializer" function

        <b>Input data:</b>
        "instance" Enum object, "attribute" Attribute object, "value" parameter

        <b>Expected results:</b>
        Test passes if value returned by "attr_enum_to_str_serializer" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "attr_enum_to_str_serializer" function when "Enum" object is specified as "value"
        parameter
        2. Check value returned by "attr_enum_to_str_serializer" function when "str" object is specified as "value"
        parameter
        """
        # Checking value returned by "attr_enum_to_str_serializer" when "Enum" object is specified as "value"
        assert (
            attr_enum_to_str_serializer(
                instance=ElementCategory,
                attribute=ElementCategory.PRIMITIVES.name,
                value=ElementCategory.PRIMITIVES,
            )
            == "PRIMITIVES"
        )
        assert (
            attr_enum_to_str_serializer(
                instance=ElementCategory,
                attribute=ElementCategory.RULES.name,
                value=ElementCategory.RULES,
            )
            == "RULES"
        )
        # Checking value returned by "attr_enum_to_str_serializer" when "str" object is specified as "value"
        assert (
            attr_enum_to_str_serializer(
                instance=ElementCategory,
                attribute=self.attribute.id,
                value="non enum string",
            )
            == "non enum string"
        )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_convert_enum_selectable_value(self):
        """
        <b>Description:</b>
        Check "_convert_enum_selectable_value" function

        <b>Input data:</b>
        "value" parameter, "enum_class" ConfigurableEnum object

        <b>Expected results:</b>
        Test passes if ConfigurableEnum class element returned by "_convert_enum_selectable_value" function is
        equal to expected

        <b>Steps</b>
        1. Check ConfigurableEnum class element returned by "_convert_enum_selectable_value" function when string
        is specified as "value" parameter
        2. Check ConfigurableEnum class element returned by "_convert_enum_selectable_value" function when
        ConfigurableEnum class element is specified as "value" parameter
        3. Check that ValueError exception is raised by "_convert_enum_selectable_value" function when unexpected string
        is specified as "value" parameter
        """
        # Checking ConfigurableEnum element returned by "_convert_enum_selectable_value" when string is specified as
        # "value"
        assert (
            _convert_enum_selectable_value(value="test_2_test", enum_class=SomeEnumSelectable)
            == SomeEnumSelectable.TEST_2
        )
        # Checking ConfigurableEnum element returned by "_convert_enum_selectable_value" when ConfigurableEnum element
        # is specified as "value"
        assert (
            _convert_enum_selectable_value(value=SomeEnumSelectable.OPTION_C, enum_class=SomeEnumSelectable)
            == SomeEnumSelectable.OPTION_C
        )
        # Checking that ValueError exception is raised by "_convert_enum_selectable_value" when unexpected string is
        # specified as "value"
        with pytest.raises(ValueError):
            _convert_enum_selectable_value(value="some string", enum_class=SomeEnumSelectable)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_construct_attr_enum_selectable_converter(self):
        """
        <b>Description:</b>
        Check "construct_attr_enum_selectable_converter" function

        <b>Input data:</b>
        "default_value" ConfigurableEnum element

        <b>Expected results:</b>
        Test passes if function returned by "construct_attr_enum_selectable_converter" function is equal to expected
        """
        converter = construct_attr_enum_selectable_converter(default_value=SomeEnumSelectable.TEST_NAME1)
        assert isinstance(converter, FunctionType)
        assert converter(SomeEnumSelectable.BOGUS_NAME) == SomeEnumSelectable.BOGUS_NAME
        assert converter("test_2_test") == SomeEnumSelectable.TEST_2

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_construct_attr_enum_selectable_onsetattr(self):
        """
        <b>Description:</b>
        Check "construct_attr_enum_selectable_onsetattr" function

        <b>Input data:</b>
        "default_value" ConfigurableEnum element, "parameter_group" ParameterGroup object, "attribute" Attribute object,
        "value" parameter

        <b>Expected results:</b>
        Test passes if function returned by "construct_attr_enum_selectable_onsetattr" function is equal to expected
        """
        on_set_attr = construct_attr_enum_selectable_onsetattr(default_value=SomeEnumSelectable.TEST_2)
        assert isinstance(on_set_attr, FunctionType)
        assert (
            on_set_attr(
                self.parameter_group,
                SomeEnumSelectable.TEST_2.value,
                SomeEnumSelectable.TEST_2,
            )
            == SomeEnumSelectable.TEST_2
        )
        assert (
            on_set_attr(self.parameter_group, SomeEnumSelectable.OPTION_C.value, "option_c")
            == SomeEnumSelectable.OPTION_C
        )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_construct_attr_value_validator(self):
        """
        <b>Description:</b>
        Check "construct_attr_value_validator" function

        <b>Input data:</b>
        "min_value" and "max_value" parameters, "parameter_group" ParameterGroup object, "attribute" Attribute object,
        "value" parameter

        <b>Expected results:</b>
        Test passes if validator returned by "construct_attr_value_validator" function is equal to expected

        <b>Steps</b>
        1. Check that ValueError exception is not raised by validator returned by "construct_attr_value_validator"
        function for values within specified bounds
        2. Check that ValueError exception is raised by validator returned by "construct_attr_value_validator" function
        for values out of specified bounds
        """
        attr_value_validator = construct_attr_value_validator(min_value=1, max_value=4)
        # Checking that ValueError exception is not raised by validator returned by "construct_attr_value_validator"
        # for values within specified bounds
        for value_within in range(1, 5):
            attr_value_validator(self.parameter_group, self.attribute.id, value_within)
        # Checking that ValueError exception is raised by validator returned by "construct_attr_value_validator" for
        # values out of specified bounds
        for out_of_bounds_value in [0, 5]:
            with pytest.raises(ValueError):
                attr_value_validator(self.parameter_group, self.attribute.id, out_of_bounds_value)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_construct_attr_selectable_validator(self):
        """
        <b>Description:</b>
        Check "construct_attr_selectable_validator" function

        <b>Input data:</b>
        "options" list

        <b>Expected results:</b>
        Test passes if validator returned by "construct_attr_selectable_validator" function is equal to expected

        <b>Steps</b>
        1. Check that ValueError exception is not raised by validator returned by "construct_attr_selectable_validator"
        function for values included in "options" list
        2. Check that ValueError exception is raised by validator returned by "construct_attr_selectable_validator"
        function for values not included in "options" list
        """
        attr_selectable_validator = construct_attr_selectable_validator(options=["str_option", 2])
        # Checking that ValueError exception is not raised by validator returned by
        # "construct_attr_selectable_validator" for values included in "options"
        for value_within in ["str_option", 2]:
            attr_selectable_validator(self.parameter_group, self.attribute.id, value_within)
        # Checking that ValueError exception is raised by validator returned by "construct_attr_selectable_validator"
        # for values not included in "options"
        for out_of_bounds_value in ["other_str_option", 3]:
            with pytest.raises(ValueError):
                attr_selectable_validator(self.parameter_group, self.attribute.id, out_of_bounds_value)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_convert_string_to_id(self):
        """
        <b>Description:</b>
        Check "convert_string_to_id" function

        <b>Input data:</b>
        "id_string" string or ID object

        <b>Expected results:</b>
        Test passes if ID object returned by "convert_string_to_id" function is equal to expected

        <b>Steps</b>
        1. Check ID object returned by "convert_string_to_id" function for string "id_string" parameter
        2. Check ID object returned by "convert_string_to_id" function for ID "id_string" parameter
        3. Check ID object returned by "convert_string_to_id" function for "id_string" parameter equal to None
        4. Check ID object returned by "convert_string_to_id" function for int "id_string" parameter
        """
        # Checking ID returned by "convert_string_to_id" for string "id_string"
        assert convert_string_to_id("some_id") == ID("some_id")
        # Checking ID returned by "convert_string_to_id" for ID "id_string"
        assert convert_string_to_id(ID("id_string")) == ID("id_string")
        # Checking ID returned by "convert_string_to_id" for "id_string" equal to None
        assert convert_string_to_id(None) == ID()
        # Checking ID returned by "convert_string_to_id" for int "id_string"
        assert convert_string_to_id(4) == 4  # type: ignore

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_attr_strict_int_validator(self):
        """
        <b>Description:</b>
        Check "attr_strict_int_validator" function

        <b>Input data:</b>
        "parameter_group" ParameterGroup object, "attribute" Attribute object, "value" parameter

        <b>Expected results:</b>
        Test passes if "attr_strict_int_validator" function raises TypeError exception for non-int "value"
        parameter

        <b>Steps</b>
        1. Check that "attr_strict_int_validator" function not raises TypeError exception for int "value" parameter
        2. Check that "attr_strict_int_validator" function raises TypeError exception for bool "value" parameter
        3. Check that "attr_strict_int_validator" function raises TypeError exception for string "value" parameter
        """
        # Checking that "attr_strict_int_validator" not raises TypeError exception for int "value"
        attr_strict_int_validator(instance=self.parameter_group, attribute=self.attribute.id, value=1)
        # Checking that "attr_strict_int_validator" raises TypeError exception for bool "value"
        with pytest.raises(TypeError):
            attr_strict_int_validator(instance=self.parameter_group, attribute=self.attribute.id, value=True)
        # Checking that "attr_strict_int_validator" raises TypeError exception for string "value"
        with pytest.raises(TypeError):
            attr_strict_int_validator(
                instance=self.parameter_group,
                attribute=self.attribute.id,
                value="some string",  # type: ignore
            )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_validate_and_convert_float(self):
        """
        <b>Description:</b>
        Check "_validate_and_convert_float" function

        <b>Input data:</b>
        "value" parameter

        <b>Expected results:</b>
        Test passes if value returned by "_validate_and_convert_float" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "_validate_and_convert_float" function for float "value" parameter
        2. Check value returned by "_validate_and_convert_float" function for int "value" parameter
        3. Check value returned by "_validate_and_convert_float" function for bool "value" parameter
        4. Check value returned by "_validate_and_convert_float" function for str "value" parameter
        """
        # Checking value returned by "_validate_and_convert_float" for float "value"
        assert _validate_and_convert_float(value=1.3) == 1.3
        # Checking value returned by "_validate_and_convert_float" for int "value"
        converted_value = _validate_and_convert_float(value=2)
        assert isinstance(converted_value, float)
        assert converted_value == float(2)
        # Checking value returned by "_validate_and_convert_float" for bool "value"
        assert not _validate_and_convert_float(value=True)
        assert not _validate_and_convert_float(value=False)
        # Checking value returned by "_validate_and_convert_float" for str "value"
        assert not _validate_and_convert_float(value="some string")  # type: ignore

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_attr_strict_float_on_setattr(self):
        """
        <b>Description:</b>
        Check "attr_strict_float_on_setattr" function

        <b>Input data:</b>
        "parameter_group" ParameterGroup object, "attribute" Attribute object, "value" parameter

        <b>Expected results:</b>
        Test passes if value returned by "attr_strict_float_on_setattr" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "attr_strict_float_on_setattr" function for float "value" parameter
        2. Check value returned by "attr_strict_float_on_setattr" function for int "value" parameter
        3. Check that "attr_strict_float_on_setattr" function raises TypeError exception for bool "value" parameter
        4. Check that "attr_strict_float_on_setattr" function raises TypeError exception for str "value" parameter
        """
        # Checking value returned by "attr_strict_float_on_setattr" for float "value"
        assert (
            attr_strict_float_on_setattr(instance=self.parameter_group, attribute=self.attribute.id, value=10.7) == 10.7
        )
        # Checking value returned by "attr_strict_float_on_setattr" for int "value"
        converted_value = attr_strict_float_on_setattr(
            instance=self.parameter_group, attribute=self.attribute.id, value=2
        )
        assert isinstance(converted_value, float)
        assert converted_value == float(2)
        # Checking that "attr_strict_float_on_setattr" raises TypeError exception for bool "value"
        with pytest.raises(TypeError):
            attr_strict_float_on_setattr(instance=self.parameter_group, attribute=self.attribute.id, value=True)
        # Checking that "attr_strict_float_on_setattr" raises TypeError exception for str "value"
        with pytest.raises(TypeError):
            attr_strict_float_on_setattr(
                instance=self.parameter_group,
                attribute=self.attribute.id,
                value="some string",  # type: ignore
            )

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_attr_strict_float_converter(self):
        """
        <b>Description:</b>
        Check "attr_strict_float_converter" function

        <b>Input data:</b>
        "value" parameter

        <b>Expected results:</b>
        Test passes if value returned by "attr_strict_float_converter" function is equal to expected

        <b>Steps</b>
        1. Check value returned by "attr_strict_float_converter" function for float "value" parameter
        2. Check value returned by "attr_strict_float_converter" function for int "value" parameter
        3. Check value returned by "attr_strict_float_converter" function for bool or str "value" parameter
        """
        # Checking value returned by "attr_strict_float_converter" for float "value"
        assert attr_strict_float_converter(value=20.1) == 20.1
        # Checking value returned by "attr_strict_float_converter" for int "value"
        converted_value = attr_strict_float_converter(value=0)
        assert isinstance(converted_value, float)
        assert converted_value == float(0)
        # Checking that "attr_strict_float_converter" raises TypeError for bool or str "value"
        for non_bool_value in [True, False, "some string"]:
            with pytest.raises(TypeError):
                attr_strict_float_converter(non_bool_value)  # type: ignore



"""EMA hooks."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import math
from math import cos, pi

from mmcv.parallel import is_module_wrapper
from mmcv.runner import HOOKS, BaseRunner, Hook
from mmcv.runner.hooks.ema import EMAHook

from otx.algorithms.common.utils.logger import get_logger

logger = get_logger()


@HOOKS.register_module()
class CustomModelEMAHook(EMAHook):
    """Custom EMAHook to update momentum for ema over training."""

    def __init__(self, momentum=0.0002, epoch_momentum=0.0, interval=1, **kwargs):
        super().__init__(momentum=momentum, interval=interval, **kwargs)
        self.momentum = momentum
        self.epoch_momentum = epoch_momentum
        self.interval = interval

    def before_train_epoch(self, runner):
        """Update the momentum."""
        if self.epoch_momentum > 0.0:
            iter_per_epoch = len(runner.data_loader)
            epoch_decay = 1 - self.epoch_momentum
            iter_decay = math.pow(epoch_decay, self.interval / iter_per_epoch)
            self.momentum = 1 - iter_decay
            logger.info(f"Update EMA momentum: {self.momentum}")
            self.epoch_momentum = 0.0  # disable re-compute

        super().before_train_epoch(runner)


@HOOKS.register_module()
class EMAMomentumUpdateHook(Hook):
    """Exponential moving average (EMA) momentum update hook for self-supervised methods.

    This hook includes momentum adjustment in self-supervised methods following:
        m = 1 - ( 1- m_0) * (cos(pi * k / K) + 1) / 2,
        k: current step, K: total steps.

    :param end_momentum: The final momentum coefficient for the target network, defaults to 1.
    :param update_interval: Interval to update new momentum, defaults to 1.
    :param by_epoch: Whether updating momentum by epoch or not, defaults to False.
    """

    def __init__(self, end_momentum: float = 1.0, update_interval: int = 1, by_epoch: bool = False):
        self.by_epoch = by_epoch
        self.end_momentum = end_momentum
        self.update_interval = update_interval

    def before_train_epoch(self, runner: BaseRunner):
        """Called before_train_epoch in EMAMomentumUpdateHook."""
        if not self.by_epoch:
            return

        if is_module_wrapper(runner.model):
            model = runner.model.module
        else:
            model = runner.model

        if not hasattr(model, "momentum"):
            raise AttributeError('The model must have attribute "momentum".')
        if not hasattr(model, "base_momentum"):
            raise AttributeError('The model must have attribute "base_momentum".')

        if self.every_n_epochs(runner, self.update_interval):
            cur_epoch = runner.epoch
            max_epoch = runner.max_epochs
            base_m = model.base_momentum
            updated_m = (
                self.end_momentum - (self.end_momentum - base_m) * (cos(pi * cur_epoch / float(max_epoch)) + 1) / 2
            )
            model.momentum = updated_m

    def before_train_iter(self, runner: BaseRunner):
        """Called before_train_iter in EMAMomentumUpdateHook."""
        if self.by_epoch:
            return

        if is_module_wrapper(runner.model):
            model = runner.model.module
        else:
            model = runner.model

        if not hasattr(model, "momentum"):
            raise AttributeError('The model must have attribute "momentum".')
        if not hasattr(model, "base_momentum"):
            raise AttributeError('The model must have attribute "base_momentum".')

        if self.every_n_iters(runner, self.update_interval):
            cur_iter = runner.iter
            max_iter = runner.max_iters
            base_m = model.base_momentum
            updated_m = (
                self.end_momentum - (self.end_momentum - base_m) * (cos(pi * cur_iter / float(max_iter)) + 1) / 2
            )
            model.momentum = updated_m

    def after_train_iter(self, runner: BaseRunner):
        """Called after_train_iter in EMAMomentumUpdateHook."""
        if self.every_n_iters(runner, self.update_interval):
            if is_module_wrapper(runner.model):
                runner.model.module.momentum_update()
            else:
                runner.model.momentum_update()



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-Lggggcense-Identifier: Apache-2.0
#

import pytest
import torch

from otx.core.ov.ops.builder import OPS
from otx.core.ov.ops.modules.op_module import OperationModule
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestOperationModule:
    @e2e_pytest_unit
    def test(sefl):

        constant_cls = OPS.get_by_name("ConstantV0")
        constant = constant_cls(
            "weight",
            data=torch.tensor([1.5]),
            shape=(1,),
            is_parameter=True,
        )

        multiply_cls = OPS.get_by_name("MultiplyV1")
        multiply = multiply_cls("multiply", shape=(1,))

        op = OperationModule(multiply, [None, constant])
        assert op.type == multiply.type
        assert op.version == multiply.version
        assert op.name == multiply.name
        assert op.shape == multiply.shape
        assert op.attrs == multiply.attrs
        assert torch.equal(op(torch.tensor([1.0])), torch.tensor([1.5]))
        assert torch.equal(op(input_0=torch.tensor([1.0])), torch.tensor([1.5]))
        with pytest.raises(ValueError):
            op(input_1=torch.tensor([1.0]))

        op = OperationModule(multiply, {"input_0": None, "input_1": constant})
        assert op.type == multiply.type
        assert op.version == multiply.version
        assert op.name == multiply.name
        assert op.shape == multiply.shape
        assert op.attrs == multiply.attrs
        assert torch.equal(op(torch.tensor([1.0])), torch.tensor([1.5]))
        assert torch.equal(op(input_0=torch.tensor([1.0])), torch.tensor([1.5]))
        with pytest.raises(ValueError):
            op(input_1=torch.tensor([1.0]))



"""Test otx segmentation task."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os

import numpy as np
import pytest

from otx.algorithms.segmentation.task import OTXSegmentationTask
from otx.api.configuration.helper import create
from otx.api.entities.inference_parameters import InferenceParameters
from otx.api.entities.model_template import (
    parse_model_template,
)
from tests.unit.algorithms.segmentation.test_helpers import (
    DEFAULT_SEG_TEMPLATE_DIR,
    generate_otx_dataset,
    generate_otx_label_schema,
    init_environment,
)
from otx.api.usecases.tasks.interfaces.export_interface import ExportType
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class MockOTXSegmentationTask(OTXSegmentationTask):
    def _infer_model(*args, **kwargs):
        return dict(
            classes=["background", "rectangle", "ellipse", "triangle"],
            eval_predictions=[[np.random.rand(4, 128, 128)]],
            feature_vectors=[np.random.rand(600, 1, 1)],
        )

    def _train_model(*args, **kwargs):
        return {"final_ckpt": "dummy.pth"}

    def _explain_model(*args, **kwargs):
        pass

    def _export_model(*args, **kwargs):
        return {"outputs": {"bin": f"/tmp/model.xml", "xml": f"/tmp/model.bin", "onnx": f"/tmp/model.onnx"}}


class MockModel:
    class _Configuration:
        def __init__(self, label_schema):
            self.label_schema = label_schema

        def get_label_schema(self):
            return self.label_schema

    def __init__(self):
        self.model_adapters = ["weights.pth"]
        self.data = np.ndarray(1)

        label_schema = generate_otx_label_schema()

        self.configuration = self._Configuration(label_schema)

    def get_data(self, name):
        return self.data

    def set_data(self, *args, **kwargs):
        return


class TestOTXSegmentationTask:
    @pytest.fixture(autouse=True)
    def setup(self):
        model_template = parse_model_template(os.path.join(DEFAULT_SEG_TEMPLATE_DIR, "template.yaml"))
        hyper_parameters = create(model_template.hyper_parameters.data)
        task_env = init_environment(hyper_parameters, model_template)

        self.seg_task = MockOTXSegmentationTask(task_env)

    @e2e_pytest_unit
    def test_load_model_ckpt(self, mocker):
        mocker_torch_load = mocker.patch("torch.load")
        self.seg_task._load_model_ckpt(MockModel())
        mocker_torch_load.assert_called_once()

    @e2e_pytest_unit
    def test_train(self, mocker):
        dataset = generate_otx_dataset(5)
        mocker.patch("torch.load", return_value=np.ndarray([1]))
        self.seg_task.train(dataset, MockModel())
        assert self.seg_task._model_ckpt == "dummy.pth"

    @e2e_pytest_unit
    def test_infer(self):
        dataset = generate_otx_dataset(5)
        predicted_dataset = self.seg_task.infer(
            dataset.with_empty_annotations(), inference_parameters=InferenceParameters(is_evaluation=False)
        )
        assert predicted_dataset[0].annotation_scene.annotations[0]

    @e2e_pytest_unit
    def test_evaluate(self, mocker):
        class _MockScoreMetric:
            def __init__(self, value):
                self.value = value

        class _MockMetric:
            def __init__(self):
                self.overall_dice = _MockScoreMetric(1.0)

            def get_performance(self):
                return 1.0

        class _MockResultEntity:
            performance = 0.0

        mocker.patch(
            "otx.algorithms.segmentation.task.MetricsHelper.compute_dice_averaged_over_pixels",
            return_value=_MockMetric(),
        )

        _result_entity = _MockResultEntity()
        self.seg_task.evaluate(_result_entity)
        assert _result_entity.performance == 1.0

    @e2e_pytest_unit
    def test_export(self, otx_model, mocker):
        mocker_open = mocker.patch("builtins.open")
        mocker_open.__enter__.return_value = True
        self.seg_task.export(ExportType.OPENVINO, otx_model)
        mocker_open.assert_called()



import pytest
from otx.algorithms.segmentation.adapters.mmseg.utils import build_scalar_scheduler, build_segmentor
from tests.test_suite.e2e_test_system import e2e_pytest_unit


@e2e_pytest_unit
def test_build_scalar_scheduler(mocker):
    cfg = mocker.MagicMock()
    builder = mocker.patch("mmseg.models.builder.MODELS.build", return_value=True)
    build_scalar_scheduler(cfg)
    builder.assert_called_once_with(cfg)


@e2e_pytest_unit
def test_build_segmentor(mocker):
    from mmcv.utils import Config

    cfg = Config({"model": {}, "load_from": "foo.pth"})
    mocker.patch("mmseg.models.build_segmentor")
    load_ckpt = mocker.patch("otx.algorithms.segmentation.adapters.mmseg.utils.builder.load_checkpoint")
    build_segmentor(cfg)
    load_ckpt.assert_called()

    build_segmentor(cfg, is_training=True)
    load_ckpt.assert_called()
    assert cfg.load_from is None



import pytest
import torch

from otx.algorithms.detection.adapters.mmdet.hooks.det_class_probability_map_hook import (
    DetClassProbabilityMapHook,
)
from otx.algorithms.detection.adapters.mmdet.models.heads.custom_atss_head import (
    CustomATSSHead,
)
from otx.algorithms.detection.adapters.mmdet.models.heads.custom_ssd_head import (
    CustomSSDHead,
)
from otx.algorithms.detection.adapters.mmdet.models.heads.custom_vfnet_head import (
    CustomVFNetHead,
)
from otx.algorithms.detection.adapters.mmdet.models.heads.custom_yolox_head import (
    CustomYOLOXHead,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestDetClassProbabilityMapHook:
    """Test class for DetClassProbabilityMapHook."""

    @pytest.fixture(autouse=True)
    def setup(self) -> None:
        class _MockModule(torch.nn.Module):
            def __init__(self) -> None:
                super().__init__()
                self.with_neck = True
                self.neck = torch.nn.Module()
                self.neck.forward = self.forward
                self.bbox_head = torch.nn.Module()
                self.bbox_head.cls_out_channels = 3

            def forward(self, x):
                return x

        self.module = _MockModule()
        self.hook = DetClassProbabilityMapHook(self.module)

    @e2e_pytest_unit
    def test_func(self, mocker) -> None:
        """Test func function."""

        mocker.patch.object(
            DetClassProbabilityMapHook, "_get_cls_scores_from_feature_map", return_value=[torch.randn(1, 3, 14, 14)]
        )
        assert self.hook.func(torch.randn(1, 3, 14, 14)) is not None

    @e2e_pytest_unit
    def test_get_cls_scores_from_feature_map_atss(self) -> None:
        """Test _get_cls_scores_from_feature_map function."""

        self.module.bbox_head = CustomATSSHead(num_classes=3, in_channels=64)
        self.hook = DetClassProbabilityMapHook(self.module)
        assert self.hook._get_cls_scores_from_feature_map(torch.Tensor(1, 3, 64, 32, 32)) is not None

    @e2e_pytest_unit
    def test_get_cls_scores_from_feature_map_yolox(self) -> None:
        """Test _get_cls_scores_from_feature_map function."""

        self.module.bbox_head = CustomYOLOXHead(num_classes=3, in_channels=64)
        self.hook = DetClassProbabilityMapHook(self.module)
        assert self.hook._get_cls_scores_from_feature_map(torch.Tensor(1, 3, 64, 32, 32)) is not None

    @e2e_pytest_unit
    def test_get_cls_scores_from_feature_map_vfnet(self) -> None:
        """Test _get_cls_scores_from_feature_map function."""

        self.module.bbox_head = CustomVFNetHead(num_classes=3, in_channels=64)
        self.module.bbox_head.anchor_generator.num_base_anchors = 1
        self.hook = DetClassProbabilityMapHook(self.module)
        assert self.hook._get_cls_scores_from_feature_map(torch.Tensor(1, 3, 64, 32, 32)) is not None

    @e2e_pytest_unit
    def test_get_cls_scores_from_feature_map_ssd(self) -> None:
        """Test _get_cls_scores_from_feature_map function."""

        self.module.bbox_head = CustomSSDHead(
            anchor_generator=dict(
                type="SSDAnchorGenerator",
                basesize_ratio_range=(0.15, 0.9),
                strides=(16, 32, 48),
                ratios=[[0.5], [0.1], [0.3]],
            ),
            act_cfg={},
        )
        self.hook = DetClassProbabilityMapHook(self.module)
        assert self.hook._get_cls_scores_from_feature_map(torch.Tensor(1, 3, 512, 32, 32)) is not None

    @e2e_pytest_unit
    def test_get_cls_scores_from_feature_map_not_implemented_head(self) -> None:
        """Test _get_cls_scores_from_feature_map function."""

        self.module.bbox_head = torch.nn.Module()
        self.module.bbox_head.cls_out_channels = 3
        self.hook = DetClassProbabilityMapHook(self.module)
        with pytest.raises(NotImplementedError):
            self.hook._get_cls_scores_from_feature_map(torch.Tensor(1, 3, 512, 32, 32))



# Copyright (C) 2021 Intel Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions
# and limitations under the License.

import pytest
from shapely.geometry.polygon import Polygon

from otx.api.entities.shapes.ellipse import Ellipse
from otx.api.entities.shapes.rectangle import Rectangle
from otx.api.utils.time_utils import now
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestEllipse:
    modification_date = now()

    def ellipse_params(self):
        ellipse_params = {
            "x1": 0.5,
            "x2": 1.0,
            "y1": 0.0,
            "y2": 0.5,
            "modification_date": self.modification_date,
        }
        return ellipse_params

    def ellipse(self):
        return Ellipse(**self.ellipse_params())

    @staticmethod
    def width_gt_height_ellipse_params():
        width_gt_height_ellipse_params = {
            "x1": 0.5,
            "x2": 0.8,
            "y1": 0.1,
            "y2": 0.3,
        }
        return width_gt_height_ellipse_params

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse(self):
        """
        <b>Description:</b>
        Check ellipse parameters

        <b>Input data:</b>
        Coordinates

        <b>Expected results:</b>
        Test passes if Ellipse correctly calculates parameters and returns default values

        <b>Steps</b>
        1. Check ellipse params
        2. Check ellipse default values
        3. Check ellipse with incorrect coordinates
        """

        ellipse = self.ellipse()
        modification_date = self.modification_date

        assert ellipse.width == 0.5
        assert ellipse.height == 0.5
        assert ellipse.x_center == 0.75
        assert ellipse.y_center == 0.25
        assert ellipse.minor_axis == 0.25
        assert ellipse.major_axis == 0.25
        assert ellipse.modification_date == modification_date

        incorrect_ellipse_params = {
            "x1": 0,
            "x2": 0,
            "y1": 0,
            "y2": 0,
        }

        with pytest.raises(ValueError):
            Ellipse(**incorrect_ellipse_params)

        width_lt_height_ellipse_params = {
            "x1": 0.4,
            "x2": 0.5,
            "y1": 0.3,
            "y2": 0.4,
        }

        width_lt_height_ellipse = Ellipse(**width_lt_height_ellipse_params)
        assert width_lt_height_ellipse.height > width_lt_height_ellipse.width
        assert width_lt_height_ellipse.major_axis == width_lt_height_ellipse.height / 2
        assert width_lt_height_ellipse.width == pytest.approx(0.09999999999999998)
        assert width_lt_height_ellipse.height == pytest.approx(0.10000000000000003)
        assert width_lt_height_ellipse.x_center == pytest.approx(0.45)
        assert width_lt_height_ellipse.y_center == pytest.approx(0.35)
        assert width_lt_height_ellipse.minor_axis == pytest.approx(0.04999999999999999)
        assert width_lt_height_ellipse.major_axis == pytest.approx(0.05000000000000002)

        width_gt_height_ellipse = Ellipse(**self.width_gt_height_ellipse_params())
        assert width_gt_height_ellipse.height < width_gt_height_ellipse.width
        assert width_gt_height_ellipse.minor_axis == width_gt_height_ellipse.height / 2
        assert width_gt_height_ellipse.width == pytest.approx(0.30000000000000004)
        assert width_gt_height_ellipse.height == pytest.approx(0.19999999999999998)
        assert width_gt_height_ellipse.x_center == pytest.approx(0.65)
        assert width_gt_height_ellipse.y_center == pytest.approx(0.2)
        assert width_gt_height_ellipse.minor_axis == pytest.approx(0.09999999999999999)
        assert width_gt_height_ellipse.major_axis == pytest.approx(0.15000000000000002)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_magic_methods(self):
        """
        <b>Description:</b>
        Check Ellipse __repr__, __eq__, __hash__ methods

        <b>Input data:</b>
        Initialized instance of Ellipse

        <b>Expected results:</b>
        Test passes if Ellipse magic methods returns correct values

        <b>Steps</b>
        1. Initialize Ellipse instance
        2. Check returning value of magic methods
        """

        x1 = self.ellipse_params()["x1"]
        x2 = self.ellipse_params()["x2"]
        y1 = self.ellipse_params()["y1"]
        y2 = self.ellipse_params()["y2"]

        ellipse = self.ellipse()
        assert repr(ellipse) == f"Ellipse(x1={x1}, y1={y1}, x2={x2}, y2={y2})"

        other_ellipse_params = {
            "x1": 0.5,
            "x2": 1.0,
            "y1": 0.0,
            "y2": 0.5,
            "modification_date": self.modification_date,
        }

        third_ellipse_params = {
            "x1": 0.3,
            "y1": 0.5,
            "x2": 0.4,
            "y2": 0.6,
            "modification_date": self.modification_date,
        }

        other_ellipse = Ellipse(**other_ellipse_params)
        third_ellipse = Ellipse(**third_ellipse_params)

        assert ellipse == other_ellipse
        assert ellipse != third_ellipse
        assert ellipse != str

        assert hash(ellipse) == hash(str(ellipse))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_normalize_wrt_roi_shape(self):
        """
        <b>Description:</b>
        Check Ellipse normalize_wrt_roi_shape methods

        <b>Input data:</b>
        Initialized instance of Ellipse
        Initialized instance of Rectangle

        <b>Expected results:</b>
        Test passes if Ellipse normalize_wrt_roi_shape returns correct values

        <b>Steps</b>
        1. Initialize Ellipse instance
        2. Check returning value
        """

        ellipse = self.ellipse()
        roi = Rectangle(x1=0.0, x2=0.5, y1=0.0, y2=0.5)
        normalized = ellipse.normalize_wrt_roi_shape(roi)
        assert normalized.x1 == 0.25
        assert normalized.y1 == 0.0
        assert normalized.x2 == 0.5
        assert normalized.y2 == 0.25

        with pytest.raises(ValueError):
            ellipse.normalize_wrt_roi_shape("123")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_denormalize_wrt_roi_shape(self):
        """
        <b>Description:</b>
        Check Ellipse denormalize_wrt_roi_shape methods

        <b>Input data:</b>
        Initialized instance of Ellipse
        Initialized instance of Rectangle

        <b>Expected results:</b>
        Test passes if Ellipse denormalize_wrt_roi_shape returns correct values

        <b>Steps</b>
        1. Initialize Ellipse instance
        2. Check returning value
        """

        ellipse = self.ellipse()
        roi = Rectangle(x1=0.5, x2=1.0, y1=0.0, y2=1.0)
        denormalized = ellipse.denormalize_wrt_roi_shape(roi)
        assert denormalized.x1 == 0.0
        assert denormalized.y1 == 0.0
        assert denormalized.x2 == 1.0
        assert denormalized.y2 == 0.5

        with pytest.raises(ValueError):
            ellipse.denormalize_wrt_roi_shape("123")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_get_evenly_distributed_ellipse_coordinates(self):
        """
        <b>Description:</b>
        Check Ellipse get_evenly_distributed_ellipse_coordinates methods

        <b>Input data:</b>
        Initialized instance of Ellipse

        <b>Expected results:</b>
        Test passes if Ellipse get_evenly_distributed_ellipse_coordinates returns correct values

        <b>Steps</b>
        1. Initialize Ellipse instance
        2. Check returning value
        """
        ellipse = self.ellipse()
        number_of_coordinates = 3
        coordinates_ellipse_line = ellipse.get_evenly_distributed_ellipse_coordinates(number_of_coordinates)
        assert len(coordinates_ellipse_line) == 3
        assert coordinates_ellipse_line[0] == pytest.approx((1.0, 0.25))
        assert coordinates_ellipse_line[1] == pytest.approx((0.625, 0.4665063509461097))
        assert coordinates_ellipse_line[2] == pytest.approx((0.6249999999999999, 0.033493649053890406))

        width_gt_height_ellipse = Ellipse(**self.width_gt_height_ellipse_params())
        coordinates_ellipse_line = width_gt_height_ellipse.get_evenly_distributed_ellipse_coordinates(
            number_of_coordinates
        )
        assert width_gt_height_ellipse.height < width_gt_height_ellipse.width
        assert len(coordinates_ellipse_line) == 3
        assert coordinates_ellipse_line[0] == pytest.approx((0.65, 0.3))
        assert coordinates_ellipse_line[1] == pytest.approx((0.7666223198362645, 0.1371094972158116))
        assert coordinates_ellipse_line[2] == pytest.approx((0.5333776801637811, 0.13710949721577403))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse__as_shapely_polygon(self):
        """
        <b>Description:</b>
        Check Ellipse _as_shapely_polygon methods

        <b>Input data:</b>
        Initialized instance of Ellipse

        <b>Expected results:</b>
        Test passes if Ellipse _as_shapely_polygon returns correct values

        <b>Steps</b>
        1. Initialize Ellipse instance
        2. Check returning value
        """

        ellipse = self.ellipse()
        shapely_polygon = ellipse._as_shapely_polygon()
        assert shapely_polygon.__class__ == Polygon
        assert shapely_polygon.area == pytest.approx(0.1958331774442254)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_ellipse_get_area(self):
        """
        <b>Description:</b>
        Check Ellipse get_area methods

        <b>Input data:</b>
        Initialized instance of Ellipse

        <b>Expected results:</b>
        Test passes if Ellipse get_area returns correct values

        <b>Steps</b>
        1. Initialize Ellipse instance
        2. Check returning value
        """

        ellipse = self.ellipse()
        area = ellipse.get_area()
        assert area == pytest.approx(0.19634954084936207)
